PRODUCTION_QUERY = '''
query AppQuery {
  repositoriesOrError {
    ... on PythonError {
      message
      stack
    }
    ... on RepositoryConnection {
      nodes {
        pipelines {
          ...PipelineFragment
        }
      }
    }
  }
}


fragment PipelineFragment on Pipeline {
  name
  description
  tags {
    key
    value
  }
  solids {
    ...SolidFragment
    __typename
  }
  modes {
    name
    description
    resources {
      name
      configField {
        ...ConfigFieldFragment
        __typename
      }
    }
  }
  ...PipelineGraphFragment
  __typename
}

fragment SolidFragment on Solid {
  ...SolidTypeSignatureFragment
  name
  definition {
    description
    metadata {
      key
      value
      __typename
    }
    ... on SolidDefinition {
      inputDefinitions {
        name
        description
        type { key }
      }
      outputDefinitions {
        name
        description
        type { key }
      }
      configField {
        ...ConfigFieldFragment
        __typename
      }
    }
    __typename
  }
  inputs {
    definition {
      name
      description
      type {
        ... DagsterTypeWithTooltipFragment
        __typename
      }
      __typename
    }
    dependsOn {
      definition {
        name
        __typename
      }
      solid {
        name
        __typename
      }
      __typename
    }
    __typename
  }
  outputs {
    definition {
      name
      description
      type {
        __typename
      }
      __typename
    }
    __typename
  }
  __typename
}

fragment DagsterTypeWithTooltipFragment on DagsterType {
  name
  description
  __typename
}

fragment SolidTypeSignatureFragment on Solid {
  outputs {
    definition {
      name
      type {
        ... DagsterTypeWithTooltipFragment
        __typename
      }
      __typename
    }
    __typename
  }
  inputs {
    definition {
      name
      type {
        ... DagsterTypeWithTooltipFragment
        __typename
      }
      __typename
    }
    __typename
  }
  __typename
}

fragment ConfigFieldFragment on ConfigTypeField {
  configType {
    __typename
    key
    description
    ... on CompositeConfigType {
      fields {
        name
        description
        isRequired
        configType {
          key
          description
          ... on CompositeConfigType {
            fields {
              name
              description
              isRequired
              configType {
                key
                description
                __typename
              }
              __typename
            }
            __typename
          }
          __typename
        }
        __typename
      }
      __typename
    }
  }
  __typename
}

fragment PipelineGraphFragment on Pipeline {
  name
  solids {
    ...SolidNodeFragment
    __typename
  }
  __typename
}

fragment SolidNodeFragment on Solid {
  name
  inputs {
    definition {
      name
      type {
        name
        __typename
      }
      __typename
    }
    dependsOn {
      definition {
        name
        __typename
      }
      solid {
        name
        __typename
      }
      __typename
    }
    __typename
  }
  outputs {
    definition {
      name
      type {
        name
        __typename
      }
      __typename
    }
    __typename
  }
  __typename
}
'''



from dagster_graphql.client.mutations import (
    execute_execute_plan_mutation,
    execute_execute_plan_mutation_raw,
)

from dagster import file_relative_path
from dagster.cli.workspace.cli_target import PythonFileTarget, workspace_from_load_target
from dagster.core.definitions.reconstructable import ReconstructablePipeline
from dagster.core.host_representation import UserProcessApi
from dagster.core.instance import DagsterInstance

EXPECTED_EVENTS = {
    ('STEP_INPUT', 'sleeper.compute'),
    ('STEP_INPUT', 'sleeper_2.compute'),
    ('STEP_INPUT', 'sleeper_3.compute'),
    ('STEP_INPUT', 'sleeper_4.compute'),
    ('STEP_INPUT', 'total.compute'),
    ('STEP_OUTPUT', 'giver.compute'),
    ('STEP_OUTPUT', 'sleeper.compute'),
    ('STEP_OUTPUT', 'sleeper_2.compute'),
    ('STEP_OUTPUT', 'sleeper_3.compute'),
    ('STEP_OUTPUT', 'sleeper_4.compute'),
    ('STEP_OUTPUT', 'total.compute'),
    ('STEP_START', 'giver.compute'),
    ('STEP_START', 'sleeper.compute'),
    ('STEP_START', 'sleeper_2.compute'),
    ('STEP_START', 'sleeper_3.compute'),
    ('STEP_START', 'sleeper_4.compute'),
    ('STEP_START', 'total.compute'),
    ('STEP_SUCCESS', 'giver.compute'),
    ('STEP_SUCCESS', 'sleeper.compute'),
    ('STEP_SUCCESS', 'sleeper_2.compute'),
    ('STEP_SUCCESS', 'sleeper_3.compute'),
    ('STEP_SUCCESS', 'sleeper_4.compute'),
    ('STEP_SUCCESS', 'total.compute'),
}


def load_sleepy_workspace():
    return workspace_from_load_target(
        PythonFileTarget(
            file_relative_path(__file__, 'sleepy.py'), 'sleepy_pipeline', UserProcessApi.CLI
        )
    )


def sleepy_recon_pipeline():
    return ReconstructablePipeline.for_file(
        file_relative_path(__file__, 'sleepy.py'), 'sleepy_pipeline'
    )


def test_execute_execute_plan_mutation():
    pipeline_name = 'sleepy_pipeline'

    pipeline = sleepy_recon_pipeline()
    workspace = load_sleepy_workspace()
    instance = DagsterInstance.local_temp()
    pipeline_run = instance.create_run_for_pipeline(pipeline_def=pipeline.get_definition())
    variables = {
        'executionParams': {
            'runConfigData': {},
            'mode': 'default',
            'selector': {
                'repositoryLocationName': pipeline_name,
                'repositoryName': '<<unnamed>>',
                'pipelineName': pipeline_name,
            },
            'executionMetadata': {'runId': pipeline_run.run_id},
        }
    }
    result = execute_execute_plan_mutation(workspace, variables, instance_ref=instance.get_ref())
    seen_events = set()
    for event in result:
        seen_events.add((event.event_type_value, event.step_key))

    assert seen_events == EXPECTED_EVENTS


def test_execute_execute_plan_mutation_raw():
    pipeline_name = 'sleepy_pipeline'
    pipeline = sleepy_recon_pipeline()
    workspace = load_sleepy_workspace()

    instance = DagsterInstance.local_temp()
    pipeline_run = instance.create_run_for_pipeline(pipeline_def=pipeline.get_definition())
    variables = {
        'executionParams': {
            'runConfigData': {},
            'mode': 'default',
            'selector': {
                'repositoryLocationName': pipeline_name,
                'repositoryName': '<<unnamed>>',
                'pipelineName': pipeline_name,
            },
            'executionMetadata': {'runId': pipeline_run.run_id},
        }
    }
    result = execute_execute_plan_mutation_raw(
        workspace, variables, instance_ref=instance.get_ref()
    )
    seen_events = set()
    for event in result:
        seen_events.add((event.dagster_event.event_type_value, event.step_key))

    assert seen_events == EXPECTED_EVENTS



import copy

from dagster_graphql.test.utils import (
    define_context_for_file,
    execute_dagster_graphql,
    infer_pipeline_selector,
)

from dagster import execute_pipeline, lambda_solid, pipeline, repository, seven
from dagster.core.definitions.executable import InMemoryExecutablePipeline
from dagster.core.execution.api import execute_run
from dagster.core.instance import DagsterInstance
from dagster.core.storage.tags import PARENT_RUN_ID_TAG, ROOT_RUN_ID_TAG

RUNS_QUERY = '''
query PipelineRunsRootQuery($selector: PipelineSelector!) {
  pipelineOrError(params: $selector) {
    ... on Pipeline {
      name
      pipelineSnapshotId
      runs {
        ...RunHistoryRunFragment
      }
    }
  }
}

fragment RunHistoryRunFragment on PipelineRun {
  runId
  status
  pipeline {
    ...on PipelineReference { name }
  }
  executionPlan {
    steps {
      key
    }
  }
  runConfigYaml
  mode
  canTerminate
}
'''

DELETE_RUN_MUTATION = '''
mutation DeleteRun($runId: String!) {
  deletePipelineRun(runId: $runId) {
    __typename
    ... on DeletePipelineRunSuccess {
      runId
    }
    ... on PythonError {
      message
    }
  }
}
'''


ALL_RUNS_QUERY = '''
{
  pipelineRunsOrError{
    ... on PipelineRuns {
      results {
        runId
        pipelineSnapshotId
        pipeline {
          __typename
          ... on PipelineReference {
            name
            solidSelection
          }
        }
      }
    }
  }
}
'''


FILTERED_RUN_QUERY = '''
query PipelineRunsRootQuery($filter: PipelineRunsFilter!) {
  pipelineRunsOrError(filter: $filter) {
    ... on PipelineRuns {
      results {
        runId
      }
    }
  }
}
'''


RUN_GROUP_QUERY = '''
query RunGroupQuery($runId: ID!) {
  runGroupOrError(runId: $runId) {
    ... on RunGroup {
      __typename
      rootRunId
      runs {
        runId
        pipeline {
          __typename
          ... on PipelineReference {
            name
          }
        }
      }
    }
    ... on RunGroupNotFoundError {
      __typename
      runId
      message
    }
  }
}
'''


ALL_RUN_GROUPS_QUERY = '''
{
  runGroupsOrError {
    results {
      rootRunId
      runs {
        runId
        pipelineSnapshotId
        pipeline {
          __typename
          ... on PipelineReference {
            name
            solidSelection
          }
        }
      }
    }
  }
}
'''


def _get_runs_data(result, run_id):
    for run_data in result.data['pipelineOrError']['runs']:
        if run_data['runId'] == run_id:
            # so caller can delete keys
            return copy.deepcopy(run_data)

    return None


def test_get_runs_over_graphql(graphql_context):
    # This include needs to be here because its inclusion screws up
    # other code in this file which reads itself to load a repo
    from .utils import sync_execute_get_run_log_data

    selector = infer_pipeline_selector(graphql_context, "multi_mode_with_resources")

    payload_one = sync_execute_get_run_log_data(
        context=graphql_context,
        variables={
            'executionParams': {
                'selector': selector,
                'mode': 'add_mode',
                'runConfigData': {'resources': {'op': {'config': 2}}},
            }
        },
    )
    run_id_one = payload_one['run']['runId']

    payload_two = sync_execute_get_run_log_data(
        context=graphql_context,
        variables={
            'executionParams': {
                'selector': selector,
                'mode': 'add_mode',
                'runConfigData': {'resources': {'op': {'config': 3}}},
            }
        },
    )

    run_id_two = payload_two['run']['runId']

    read_context = graphql_context

    result = execute_dagster_graphql(read_context, RUNS_QUERY, variables={'selector': selector})

    # delete the second run
    result = execute_dagster_graphql(
        read_context, DELETE_RUN_MUTATION, variables={'runId': run_id_two}
    )
    assert result.data['deletePipelineRun']['__typename'] == 'DeletePipelineRunSuccess'
    assert result.data['deletePipelineRun']['runId'] == run_id_two

    # query it back out
    result = execute_dagster_graphql(read_context, RUNS_QUERY, variables={'selector': selector})

    # first is the same
    run_one_data = _get_runs_data(result, run_id_one)
    assert run_one_data

    # second is gone
    run_two_data = _get_runs_data(result, run_id_two)
    assert run_two_data is None

    # try to delete the second run again
    execute_dagster_graphql(read_context, DELETE_RUN_MUTATION, variables={'runId': run_id_two})

    result = execute_dagster_graphql(
        read_context, DELETE_RUN_MUTATION, variables={'runId': run_id_two}
    )
    assert result.data['deletePipelineRun']['__typename'] == 'PipelineRunNotFoundError'


def get_repo_at_time_1():
    @lambda_solid
    def solid_A():
        pass

    @lambda_solid
    def solid_B():
        pass

    @pipeline
    def evolving_pipeline():
        solid_A()
        solid_B()

    @pipeline
    def foo_pipeline():
        solid_A()

    @repository
    def evolving_repo():
        return [evolving_pipeline, foo_pipeline]

    return evolving_repo


def get_repo_at_time_2():
    @lambda_solid
    def solid_A():
        pass

    @lambda_solid
    def solid_B_prime():
        pass

    @pipeline
    def evolving_pipeline():
        solid_A()
        solid_B_prime()

    @pipeline
    def bar_pipeline():
        solid_A()

    @repository
    def evolving_repo():
        return [evolving_pipeline, bar_pipeline]

    return evolving_repo


def test_runs_over_time():
    with seven.TemporaryDirectory() as temp_dir:
        instance = DagsterInstance.local_temp(temp_dir)

        repo_1 = get_repo_at_time_1()

        full_evolve_run_id = execute_pipeline(
            repo_1.get_pipeline('evolving_pipeline'), instance=instance
        ).run_id
        foo_run_id = execute_pipeline(repo_1.get_pipeline('foo_pipeline'), instance=instance).run_id
        evolve_a_run_id = execute_pipeline(
            repo_1.get_pipeline('evolving_pipeline').get_pipeline_subset_def({'solid_A'}),
            instance=instance,
        ).run_id
        evolve_b_run_id = execute_pipeline(
            repo_1.get_pipeline('evolving_pipeline').get_pipeline_subset_def({'solid_B'}),
            instance=instance,
        ).run_id

        context_at_time_1 = define_context_for_file(__file__, 'get_repo_at_time_1', instance)

        result = execute_dagster_graphql(context_at_time_1, ALL_RUNS_QUERY)
        assert result.data

        t1_runs = {run['runId']: run for run in result.data['pipelineRunsOrError']['results']}

        assert t1_runs[full_evolve_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'evolving_pipeline',
            'solidSelection': None,
        }

        assert t1_runs[foo_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'foo_pipeline',
            'solidSelection': None,
        }

        assert t1_runs[evolve_a_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'evolving_pipeline',
            'solidSelection': ['solid_A'],
        }

        assert t1_runs[evolve_b_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'evolving_pipeline',
            'solidSelection': ['solid_B'],
        }

        context_at_time_2 = define_context_for_file(__file__, 'get_repo_at_time_2', instance)

        result = execute_dagster_graphql(context_at_time_2, ALL_RUNS_QUERY)
        assert result.data

        t2_runs = {run['runId']: run for run in result.data['pipelineRunsOrError']['results']}

        assert t2_runs[full_evolve_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'evolving_pipeline',
            'solidSelection': None,
        }

        assert t2_runs[evolve_a_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'evolving_pipeline',
            'solidSelection': ['solid_A'],
        }
        # pipeline name changed
        assert t2_runs[foo_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'foo_pipeline',
            'solidSelection': None,
        }
        # subset no longer valid - b renamed
        assert t2_runs[evolve_b_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'evolving_pipeline',
            'solidSelection': ['solid_B'],
        }


def test_run_groups_over_time():
    with seven.TemporaryDirectory() as tempdir:
        instance = DagsterInstance.local_temp(tempdir=tempdir)

        repo_1 = get_repo_at_time_1()

        full_evolve_run_id = execute_pipeline(
            repo_1.get_pipeline('evolving_pipeline'), instance=instance
        ).run_id
        foo_run_id = execute_pipeline(repo_1.get_pipeline('foo_pipeline'), instance=instance).run_id
        evolve_a_run_id = execute_pipeline(
            repo_1.get_pipeline('evolving_pipeline').get_pipeline_subset_def({'solid_A'}),
            instance=instance,
        ).run_id
        evolve_b_run_id = execute_pipeline(
            repo_1.get_pipeline('evolving_pipeline').get_pipeline_subset_def({'solid_B'}),
            instance=instance,
        ).run_id

        context_at_time_1 = define_context_for_file(__file__, 'get_repo_at_time_1', instance)

        result = execute_dagster_graphql(context_at_time_1, ALL_RUN_GROUPS_QUERY)
        assert result.data
        assert 'runGroupsOrError' in result.data
        assert 'results' in result.data['runGroupsOrError']
        assert len(result.data['runGroupsOrError']['results']) == 4

        t1_runs = {
            run['runId']: run
            for group in result.data['runGroupsOrError']['results']
            for run in group['runs']
        }

        # test full_evolve_run_id
        assert t1_runs[full_evolve_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'evolving_pipeline',
            'solidSelection': None,
        }

        # test foo_run_id
        assert t1_runs[foo_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'foo_pipeline',
            'solidSelection': None,
        }

        # test evolve_a_run_id
        assert t1_runs[evolve_a_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'evolving_pipeline',
            'solidSelection': ['solid_A'],
        }
        assert t1_runs[evolve_a_run_id]['pipelineSnapshotId']

        # test evolve_b_run_id
        assert t1_runs[evolve_b_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'evolving_pipeline',
            'solidSelection': ['solid_B'],
        }

        context_at_time_2 = define_context_for_file(__file__, 'get_repo_at_time_2', instance)

        result = execute_dagster_graphql(context_at_time_2, ALL_RUN_GROUPS_QUERY)
        assert 'runGroupsOrError' in result.data
        assert 'results' in result.data['runGroupsOrError']
        assert len(result.data['runGroupsOrError']['results']) == 4

        t2_runs = {
            run['runId']: run
            for group in result.data['runGroupsOrError']['results']
            for run in group['runs']
        }

        # test full_evolve_run_id
        assert t2_runs[full_evolve_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'evolving_pipeline',
            'solidSelection': None,
        }

        # test evolve_a_run_id
        assert t2_runs[evolve_a_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'evolving_pipeline',
            'solidSelection': ['solid_A'],
        }
        assert t2_runs[evolve_a_run_id]['pipelineSnapshotId']

        # names same
        assert (
            t1_runs[full_evolve_run_id]['pipeline']['name']
            == t2_runs[evolve_a_run_id]['pipeline']['name']
        )

        # snapshots differ
        assert (
            t1_runs[full_evolve_run_id]['pipelineSnapshotId']
            != t2_runs[evolve_a_run_id]['pipelineSnapshotId']
        )

        # pipeline name changed
        assert t2_runs[foo_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'foo_pipeline',
            'solidSelection': None,
        }
        # subset no longer valid - b renamed
        assert t2_runs[evolve_b_run_id]['pipeline'] == {
            '__typename': 'PipelineSnapshot',
            'name': 'evolving_pipeline',
            'solidSelection': ['solid_B'],
        }


def test_filtered_runs():
    with seven.TemporaryDirectory() as temp_dir:
        instance = DagsterInstance.local_temp(temp_dir)
        repo = get_repo_at_time_1()
        run_id_1 = execute_pipeline(
            repo.get_pipeline('foo_pipeline'), instance=instance, tags={'run': 'one'}
        ).run_id
        _run_id_2 = execute_pipeline(
            repo.get_pipeline('foo_pipeline'), instance=instance, tags={'run': 'two'}
        ).run_id
        context = define_context_for_file(__file__, 'get_repo_at_time_1', instance)
        result = execute_dagster_graphql(
            context, FILTERED_RUN_QUERY, variables={'filter': {'runId': run_id_1}}
        )
        assert result.data
        run_ids = [run['runId'] for run in result.data['pipelineRunsOrError']['results']]
        assert len(run_ids) == 1
        assert run_ids[0] == run_id_1

        result = execute_dagster_graphql(
            context,
            FILTERED_RUN_QUERY,
            variables={'filter': {'tags': [{'key': 'run', 'value': 'one'}]}},
        )
        assert result.data
        run_ids = [run['runId'] for run in result.data['pipelineRunsOrError']['results']]
        assert len(run_ids) == 1
        assert run_ids[0] == run_id_1


def test_run_group():
    with seven.TemporaryDirectory() as temp_dir:
        instance = DagsterInstance.local_temp(temp_dir)
        repo = get_repo_at_time_1()
        foo_pipeline = repo.get_pipeline('foo_pipeline')
        runs = [execute_pipeline(foo_pipeline, instance=instance)]
        root_run_id = runs[-1].run_id
        for _ in range(3):
            # https://github.com/dagster-io/dagster/issues/2433
            run = instance.create_run_for_pipeline(
                foo_pipeline,
                parent_run_id=root_run_id,
                root_run_id=root_run_id,
                tags={PARENT_RUN_ID_TAG: root_run_id, ROOT_RUN_ID_TAG: root_run_id},
            )
            execute_run(InMemoryExecutablePipeline(foo_pipeline), run, instance)
            runs.append(run)

        context_at_time_1 = define_context_for_file(__file__, 'get_repo_at_time_1', instance)

        result_one = execute_dagster_graphql(
            context_at_time_1, RUN_GROUP_QUERY, variables={'runId': root_run_id},
        )
        assert result_one.data['runGroupOrError']['__typename'] == 'RunGroup'

        assert len(result_one.data['runGroupOrError']['runs']) == 4

        result_two = execute_dagster_graphql(
            context_at_time_1, RUN_GROUP_QUERY, variables={'runId': runs[-1].run_id},
        )
        assert result_one.data['runGroupOrError']['__typename'] == 'RunGroup'
        assert len(result_two.data['runGroupOrError']['runs']) == 4

        assert (
            result_one.data['runGroupOrError']['rootRunId']
            == result_two.data['runGroupOrError']['rootRunId']
        )
        assert (
            result_one.data['runGroupOrError']['runs'] == result_two.data['runGroupOrError']['runs']
        )


def test_run_group_not_found():
    with seven.TemporaryDirectory() as temp_dir:
        instance = DagsterInstance.local_temp(temp_dir)
        context_at_time_1 = define_context_for_file(__file__, 'get_repo_at_time_1', instance)

        result = execute_dagster_graphql(
            context_at_time_1, RUN_GROUP_QUERY, variables={'runId': 'foo'},
        )
        assert result.data
        assert result.data['runGroupOrError']
        assert result.data['runGroupOrError']['__typename'] == 'RunGroupNotFoundError'
        assert result.data['runGroupOrError']['runId'] == 'foo'
        assert result.data['runGroupOrError'][
            'message'
        ] == 'Run group of run {run_id} could not be found.'.format(run_id='foo')


def test_run_groups():
    with seven.TemporaryDirectory() as tempdir:
        instance = DagsterInstance.local_temp(tempdir=tempdir)

        repo = get_repo_at_time_1()
        foo_pipeline = repo.get_pipeline('foo_pipeline')

        root_run_ids = [execute_pipeline(foo_pipeline, instance=instance).run_id for i in range(3)]

        for _ in range(5):
            for root_run_id in root_run_ids:
                execute_pipeline(
                    foo_pipeline,
                    tags={PARENT_RUN_ID_TAG: root_run_id, ROOT_RUN_ID_TAG: root_run_id},
                    instance=instance,
                )

        context_at_time_1 = define_context_for_file(__file__, 'get_repo_at_time_1', instance)
        result = execute_dagster_graphql(context_at_time_1, ALL_RUN_GROUPS_QUERY)

        assert result.data
        assert 'runGroupsOrError' in result.data
        assert 'results' in result.data['runGroupsOrError']
        assert len(result.data['runGroupsOrError']['results']) == 3
        for run_group in result.data['runGroupsOrError']['results']:
            assert run_group['rootRunId'] in root_run_ids
            assert len(run_group['runs']) == 6



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots['TestAssetAwareEventLog.test_get_asset_key_materialization[in_memory_instance_in_process_env] 1'] = {
    'assetOrError': {
        'assetMaterializations': [
            {
                'materializationEvent': {
                    'materialization': {
                        'label': 'a'
                    }
                }
            }
        ]
    }
}

snapshots['TestAssetAwareEventLog.test_get_all_asset_keys[in_memory_instance_in_process_env] 1'] = {
    'assetsOrError': {
        '__typename': 'AssetConnection',
        'nodes': [
            {
                'key': {
                    'path': [
                        'a'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'b'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'c'
                    ]
                }
            }
        ]
    }
}

snapshots['TestAssetAwareEventLog.test_get_asset_key_materialization[asset_aware_instance_in_process_env] 1'] = {
    'assetOrError': {
        'assetMaterializations': [
            {
                'materializationEvent': {
                    'materialization': {
                        'label': 'a'
                    }
                }
            }
        ]
    }
}

snapshots['TestAssetAwareEventLog.test_get_all_asset_keys[asset_aware_instance_in_process_env] 1'] = {
    'assetsOrError': {
        '__typename': 'AssetConnection',
        'nodes': [
            {
                'key': {
                    'path': [
                        'a'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'b'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'c'
                    ]
                }
            }
        ]
    }
}

snapshots['TestAssetAwareEventLog.test_get_all_asset_keys[postgres_with_sync_run_launcher_in_process_env] 1'] = {
    'assetsOrError': {
        '__typename': 'AssetConnection',
        'nodes': [
            {
                'key': {
                    'path': [
                        'a'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'b'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'c'
                    ]
                }
            }
        ]
    }
}

snapshots['TestAssetAwareEventLog.test_get_asset_key_materialization[postgres_with_sync_run_launcher_in_process_env] 1'] = {
    'assetOrError': {
        'assetMaterializations': [
            {
                'materializationEvent': {
                    'materialization': {
                        'label': 'a'
                    }
                }
            }
        ]
    }
}



# NOTE: pylint isn't smart enough to figure out what's going on with dauphin
# pylint: disable=unexpected-keyword-arg, no-value-for-parameter

from dagster_graphql import dauphin

from dagster import check
from dagster.core.definitions import SolidHandle
from dagster.core.host_representation import RepresentedPipeline
from dagster.core.snap import CompositeSolidDefSnap, DependencyStructureIndex, SolidDefSnap

from .config_types import DauphinConfigTypeField
from .dagster_types import to_dauphin_dagster_type


class DauphinSolidContainer(dauphin.Interface):
    class Meta(object):
        name = 'SolidContainer'

    solids = dauphin.non_null_list('Solid')


class DauphinISolidDefinition(dauphin.Interface):
    class Meta(object):
        name = 'ISolidDefinition'

    name = dauphin.NonNull(dauphin.String)
    description = dauphin.String()
    metadata = dauphin.non_null_list('MetadataItemDefinition')
    input_definitions = dauphin.non_null_list('InputDefinition')
    output_definitions = dauphin.non_null_list('OutputDefinition')
    required_resources = dauphin.non_null_list('ResourceRequirement')


class IDauphinSolidDefinitionMixin(object):
    def __init__(self, represented_pipeline, solid_def_name):
        self._represented_pipeline = check.inst_param(
            represented_pipeline, 'represented_pipeline', RepresentedPipeline
        )
        check.str_param(solid_def_name, 'solid_def_name')
        self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)

    def resolve_metadata(self, graphene_info):
        return [
            graphene_info.schema.type_named('MetadataItemDefinition')(key=item[0], value=item[1])
            for item in self._solid_def_snap.tags.items()
        ]

    @property
    def solid_def_name(self):
        return self._solid_def_snap.name

    def resolve_input_definitions(self, _):
        return [
            DauphinInputDefinition(
                self._represented_pipeline, self.solid_def_name, input_def_snap.name
            )
            for input_def_snap in self._solid_def_snap.input_def_snaps
        ]

    def resolve_output_definitions(self, _):
        return [
            DauphinOutputDefinition(
                self._represented_pipeline, self.solid_def_name, output_def_snap.name
            )
            for output_def_snap in self._solid_def_snap.output_def_snaps
        ]

    def resolve_required_resources(self, graphene_info):
        return [
            graphene_info.schema.type_named('ResourceRequirement')(key)
            for key in self._solid_def_snap.required_resource_keys
        ]


def build_dauphin_solid_definition(represented_pipeline, solid_def_name):
    check.inst_param(represented_pipeline, 'represented_pipeline', RepresentedPipeline)
    check.str_param(solid_def_name, 'solid_def_name')

    solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)

    if isinstance(solid_def_snap, SolidDefSnap):
        return DauphinSolidDefinition(represented_pipeline, solid_def_snap.name)

    if isinstance(solid_def_snap, CompositeSolidDefSnap):
        return DauphinCompositeSolidDefinition(represented_pipeline, solid_def_snap.name)

    check.failed('Unknown solid definition type {type}'.format(type=type(solid_def_snap)))


class _ArgNotPresentSentinel:
    pass


class DauphinSolid(dauphin.ObjectType):
    class Meta(object):
        name = 'Solid'

    name = dauphin.NonNull(dauphin.String)
    definition = dauphin.NonNull('ISolidDefinition')
    inputs = dauphin.non_null_list('Input')
    outputs = dauphin.non_null_list('Output')

    def __init__(self, represented_pipeline, solid_name, current_dep_structure):
        self._represented_pipeline = check.inst_param(
            represented_pipeline, 'represented_pipeline', RepresentedPipeline
        )

        check.str_param(solid_name, 'solid_name')
        self._current_dep_structure = check.inst_param(
            current_dep_structure, 'current_dep_structure', DependencyStructureIndex
        )
        self._solid_invocation_snap = current_dep_structure.get_invocation(solid_name)
        self._solid_def_snap = represented_pipeline.get_solid_def_snap(
            self._solid_invocation_snap.solid_def_name
        )
        super(DauphinSolid, self).__init__(name=solid_name)

    def get_solid_definition_name(self):
        return self._solid_def_snap.name

    def get_dauphin_solid_definition(self):
        return build_dauphin_solid_definition(self._represented_pipeline, self._solid_def_snap.name)

    def resolve_definition(self, _):
        return self.get_dauphin_solid_definition()

    def resolve_inputs(self, _):
        return [
            DauphinInput(
                self._represented_pipeline,
                self._current_dep_structure,
                self._solid_invocation_snap.solid_name,
                input_def_snap.name,
            )
            for input_def_snap in self._solid_def_snap.input_def_snaps
        ]

    def resolve_outputs(self, _):
        return [
            DauphinOutput(
                self._represented_pipeline,
                self._current_dep_structure,
                self._solid_invocation_snap.solid_name,
                output_def_snap.name,
            )
            for output_def_snap in self._solid_def_snap.output_def_snaps
        ]


class DauphinSolidDefinition(dauphin.ObjectType, IDauphinSolidDefinitionMixin):
    class Meta(object):
        name = 'SolidDefinition'
        interfaces = [DauphinISolidDefinition]

    config_field = dauphin.Field('ConfigTypeField')

    def __init__(self, represented_pipeline, solid_def_name):
        check.inst_param(represented_pipeline, 'represented_pipeline', RepresentedPipeline)
        self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
        super(DauphinSolidDefinition, self).__init__(
            name=solid_def_name, description=self._solid_def_snap.description
        )
        IDauphinSolidDefinitionMixin.__init__(self, represented_pipeline, solid_def_name)

    def resolve_config_field(self, _):
        return (
            DauphinConfigTypeField(
                config_schema_snapshot=self._represented_pipeline.config_schema_snapshot,
                field_snap=self._solid_def_snap.config_field_snap,
            )
            if self._solid_def_snap.config_field_snap
            else None
        )


class DauphinCompositeSolidDefinition(dauphin.ObjectType, IDauphinSolidDefinitionMixin):
    class Meta(object):
        name = 'CompositeSolidDefinition'
        interfaces = [DauphinISolidDefinition, DauphinSolidContainer]

    solids = dauphin.non_null_list('Solid')
    input_mappings = dauphin.non_null_list('InputMapping')
    output_mappings = dauphin.non_null_list('OutputMapping')

    def __init__(self, represented_pipeline, solid_def_name):
        self._represented_pipeline = check.inst_param(
            represented_pipeline, 'represented_pipeline', RepresentedPipeline
        )
        self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
        self._comp_solid_dep_index = represented_pipeline.get_dep_structure_index(solid_def_name)
        super(DauphinCompositeSolidDefinition, self).__init__(
            name=solid_def_name, description=self._solid_def_snap.description
        )
        IDauphinSolidDefinitionMixin.__init__(self, represented_pipeline, solid_def_name)

    def resolve_solids(self, _graphene_info):
        return build_dauphin_solids(self._represented_pipeline, self._comp_solid_dep_index)

    def resolve_output_mappings(self, _):
        return [
            DauphinOutputMapping(
                self._represented_pipeline,
                self._comp_solid_dep_index,
                self._solid_def_snap.name,
                output_def_snap.name,
            )
            for output_def_snap in self._solid_def_snap.output_def_snaps
        ]

    def resolve_input_mappings(self, _):
        return [
            DauphinInputMapping(
                self._represented_pipeline,
                self._comp_solid_dep_index,
                self._solid_def_snap.name,
                input_def_snap.name,
            )
            for input_def_snap in self._solid_def_snap.input_def_snaps
        ]


class DauphinSolidHandle(dauphin.ObjectType):
    class Meta(object):
        name = 'SolidHandle'

    handleID = dauphin.NonNull(dauphin.String)
    solid = dauphin.NonNull('Solid')
    parent = dauphin.Field('SolidHandle')

    def __init__(self, handle, solid, parent):
        self.handleID = check.inst_param(handle, 'handle', SolidHandle)
        self.solid = check.inst_param(solid, 'solid', DauphinSolid)
        self.parent = check.opt_inst_param(parent, 'parent', DauphinSolidHandle)


class DauphinInputDefinition(dauphin.ObjectType):
    class Meta(object):
        name = 'InputDefinition'

    solid_definition = dauphin.NonNull('SolidDefinition')
    name = dauphin.NonNull(dauphin.String)
    description = dauphin.String()
    type = dauphin.NonNull('DagsterType')

    def __init__(self, represented_pipeline, solid_def_name, input_def_name):
        self._represented_pipeline = check.inst_param(
            represented_pipeline, 'represented_pipeline', RepresentedPipeline
        )
        check.str_param(solid_def_name, 'solid_def_name')
        check.str_param(input_def_name, 'input_def_name')
        solid_def_snap = self._represented_pipeline.get_solid_def_snap(solid_def_name)
        self._input_def_snap = solid_def_snap.get_input_snap(input_def_name)
        super(DauphinInputDefinition, self).__init__(
            name=self._input_def_snap.name, description=self._input_def_snap.description,
        )

    def resolve_type(self, _graphene_info):
        return to_dauphin_dagster_type(
            self._represented_pipeline.pipeline_snapshot, self._input_def_snap.dagster_type_key
        )

    def resolve_solid_definition(self, _):
        return build_dauphin_solid_definition(
            self._represented_pipeline, self._solid_def_snap.name  # pylint: disable=no-member
        )


class DauphinOutputDefinition(dauphin.ObjectType):
    class Meta(object):
        name = 'OutputDefinition'

    solid_definition = dauphin.NonNull('SolidDefinition')
    name = dauphin.NonNull(dauphin.String)
    description = dauphin.String()
    type = dauphin.NonNull('DagsterType')

    def __init__(self, represented_pipeline, solid_def_name, output_def_name):
        self._represented_pipeline = check.inst_param(
            represented_pipeline, 'represented_pipeline', RepresentedPipeline
        )
        check.str_param(solid_def_name, 'solid_def_name')
        check.str_param(output_def_name, 'output_def_name')

        self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
        self._output_def_snap = self._solid_def_snap.get_output_snap(output_def_name)

        super(DauphinOutputDefinition, self).__init__(
            name=self._output_def_snap.name, description=self._output_def_snap.description,
        )

    def resolve_type(self, _):
        return to_dauphin_dagster_type(
            self._represented_pipeline.pipeline_snapshot, self._output_def_snap.dagster_type_key,
        )

    def resolve_solid_definition(self, _):
        return build_dauphin_solid_definition(self._represented_pipeline, self._solid_def_snap.name)


class DauphinInput(dauphin.ObjectType):
    class Meta(object):
        name = 'Input'

    solid = dauphin.NonNull('Solid')
    definition = dauphin.NonNull('InputDefinition')
    depends_on = dauphin.non_null_list('Output')

    def __init__(self, represented_pipeline, current_dep_structure, solid_name, input_name):
        self._represented_pipeline = check.inst_param(
            represented_pipeline, 'represented_pipeline', RepresentedPipeline
        )
        self._current_dep_structure = check.inst_param(
            current_dep_structure, 'current_dep_structure', DependencyStructureIndex
        )
        self._solid_name = check.str_param(solid_name, 'solid_name')
        self._input_name = check.str_param(input_name, 'input_name')
        self._solid_invocation_snap = current_dep_structure.get_invocation(solid_name)
        self._solid_def_snap = represented_pipeline.get_solid_def_snap(
            self._solid_invocation_snap.solid_def_name
        )
        self._input_def_snap = self._solid_def_snap.get_input_snap(input_name)

        super(DauphinInput, self).__init__()

    def resolve_solid(self, _):
        return DauphinSolid(
            self._represented_pipeline, self._solid_name, self._current_dep_structure
        )

    def resolve_definition(self, _):
        return DauphinInputDefinition(
            self._represented_pipeline, self._solid_def_snap.name, self._input_def_snap.name,
        )

    def resolve_depends_on(self, _):
        return [
            DauphinOutput(
                self._represented_pipeline,
                self._current_dep_structure,
                output_handle_snap.solid_name,
                output_handle_snap.output_name,
            )
            for output_handle_snap in self._current_dep_structure.get_upstream_outputs(
                self._solid_name, self._input_name
            )
        ]


class DauphinOutput(dauphin.ObjectType):
    class Meta(object):
        name = 'Output'

    solid = dauphin.NonNull('Solid')
    definition = dauphin.NonNull('OutputDefinition')
    depended_by = dauphin.non_null_list('Input')

    def __init__(self, represented_pipeline, current_dep_structure, solid_name, output_name):
        self._represented_pipeline = check.inst_param(
            represented_pipeline, 'represented_pipeline', RepresentedPipeline
        )
        self._current_dep_structure = check.inst_param(
            current_dep_structure, 'current_dep_structure', DependencyStructureIndex
        )
        self._solid_name = check.str_param(solid_name, 'solid_name')
        self._output_name = check.str_param(output_name, 'output_name')
        self._solid_invocation_snap = current_dep_structure.get_invocation(solid_name)
        self._solid_def_snap = represented_pipeline.get_solid_def_snap(
            self._solid_invocation_snap.solid_def_name
        )
        self._output_def_snap = self._solid_def_snap.get_output_snap(output_name)
        super(DauphinOutput, self).__init__()

    def resolve_solid(self, _):
        return DauphinSolid(
            self._represented_pipeline, self._solid_name, self._current_dep_structure
        )

    def resolve_definition(self, _):
        return DauphinOutputDefinition(
            self._represented_pipeline, self._solid_def_snap.name, self._output_name,
        )

    def resolve_depended_by(self, _):
        return [
            DauphinInput(
                self._represented_pipeline,
                self._current_dep_structure,
                input_handle_snap.solid_name,
                input_handle_snap.input_name,
            )
            for input_handle_snap in self._current_dep_structure.get_downstream_inputs(
                self._solid_name, self._output_def_snap.name
            )
        ]


class DauphinInputMapping(dauphin.ObjectType):
    class Meta(object):
        name = 'InputMapping'

    mapped_input = dauphin.NonNull('Input')
    definition = dauphin.NonNull('InputDefinition')

    def __init__(
        self, represented_pipeline, current_dep_index, solid_def_name, input_name,
    ):
        self._represented_pipeline = check.inst_param(
            represented_pipeline, 'represented_pipeline', RepresentedPipeline
        )
        self._current_dep_index = check.inst_param(
            current_dep_index, 'current_dep_index', DependencyStructureIndex
        )
        self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
        self._input_mapping_snap = self._solid_def_snap.get_input_mapping_snap(input_name)
        super(DauphinInputMapping, self).__init__()

    def resolve_mapped_input(self, _):
        return DauphinInput(
            self._represented_pipeline,
            self._current_dep_index,
            self._input_mapping_snap.mapped_solid_name,
            self._input_mapping_snap.mapped_input_name,
        )

    def resolve_definition(self, _):
        return DauphinInputDefinition(
            self._represented_pipeline,
            self._solid_def_snap.name,
            self._input_mapping_snap.external_input_name,
        )


class DauphinOutputMapping(dauphin.ObjectType):
    class Meta(object):
        name = 'OutputMapping'

    mapped_output = dauphin.NonNull('Output')
    definition = dauphin.NonNull('OutputDefinition')

    def __init__(
        self, represented_pipeline, current_dep_index, solid_def_name, output_name,
    ):
        self._represented_pipeline = check.inst_param(
            represented_pipeline, 'represented_pipeline', RepresentedPipeline
        )
        self._current_dep_index = check.inst_param(
            current_dep_index, 'current_dep_index', DependencyStructureIndex
        )
        self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
        self._output_mapping_snap = self._solid_def_snap.get_output_mapping_snap(output_name)
        super(DauphinOutputMapping, self).__init__()

    def resolve_mapped_output(self, _):
        return DauphinOutput(
            self._represented_pipeline,
            self._current_dep_index,
            self._output_mapping_snap.mapped_solid_name,
            self._output_mapping_snap.mapped_output_name,
        )

    def resolve_definition(self, _):
        return DauphinOutputDefinition(
            self._represented_pipeline,
            self._solid_def_snap.name,
            self._output_mapping_snap.external_output_name,
        )


class DauphinResourceRequirement(dauphin.ObjectType):
    class Meta(object):
        name = 'ResourceRequirement'

    resource_key = dauphin.NonNull(dauphin.String)

    def __init__(self, resource_key):
        self.resource_key = resource_key


class DauphinUsedSolid(dauphin.ObjectType):
    class Meta(object):
        name = 'UsedSolid'
        description = '''A solid definition and it's invocations within the repo.'''

    definition = dauphin.NonNull('ISolidDefinition')
    invocations = dauphin.non_null_list('SolidInvocationSite')


class DauphinSolidInvocationSite(dauphin.ObjectType):
    class Meta(object):
        name = 'SolidInvocationSite'
        description = '''An invocation of a solid within a repo.'''

    pipeline = dauphin.NonNull('Pipeline')
    solidHandle = dauphin.NonNull('SolidHandle')


def build_dauphin_solids(represented_pipeline, current_dep_index):
    check.inst_param(represented_pipeline, 'represented_pipeline', RepresentedPipeline)
    return sorted(
        [
            DauphinSolid(represented_pipeline, solid_name, current_dep_index)
            for solid_name in current_dep_index.solid_invocation_names
        ],
        key=lambda solid: solid.name,
    )


def build_dauphin_solid_handles(represented_pipeline, current_dep_index, parent=None):
    check.inst_param(represented_pipeline, 'represented_pipeline', RepresentedPipeline)
    check.opt_inst_param(parent, 'parent', DauphinSolidHandle)
    all_handle = []
    for solid_invocation in current_dep_index.solid_invocations:
        solid_name, solid_def_name = solid_invocation.solid_name, solid_invocation.solid_def_name
        handle = DauphinSolidHandle(
            solid=DauphinSolid(represented_pipeline, solid_name, current_dep_index),
            handle=SolidHandle(solid_name, parent.handleID if parent else None),
            parent=parent if parent else None,
        )
        solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
        if isinstance(solid_def_snap, CompositeSolidDefSnap):
            all_handle += build_dauphin_solid_handles(
                represented_pipeline,
                represented_pipeline.get_dep_structure_index(solid_def_name),
                handle,
            )

        all_handle.append(handle)

    return all_handle



from dagster_graphql.implementation.context import DagsterGraphQLContext
from dagster_graphql.schema import create_schema
from graphql import graphql

from dagster import check
from dagster.core.code_pointer import FileCodePointer
from dagster.core.definitions.reconstructable import ReconstructableRepository
from dagster.core.host_representation import (
    InProcessRepositoryLocation,
    PythonEnvRepositoryLocation,
    RepositoryLocationHandle,
)
from dagster.core.instance import DagsterInstance


def execute_dagster_graphql(context, query, variables=None):
    result = graphql(
        create_schema(),
        query,
        context_value=context,
        variable_values=variables,
        allow_subscriptions=True,
        return_promise=False,
    )

    # has to check attr because in subscription case it returns AnonymousObservable
    if hasattr(result, 'errors') and result.errors:
        first_error = result.errors[0]
        if hasattr(first_error, 'original_error') and first_error.original_error:
            raise result.errors[0].original_error

        raise result.errors[0]

    return result


def execute_dagster_graphql_and_finish_runs(context, query, variables=None):
    result = execute_dagster_graphql(context, query, variables)
    context.drain_outstanding_executions()
    return result


def define_context_for_file(python_file, fn_name, instance):
    check.inst_param(instance, 'instance', DagsterInstance)
    return DagsterGraphQLContext(
        locations=[
            InProcessRepositoryLocation(ReconstructableRepository.for_file(python_file, fn_name))
        ],
        instance=instance,
    )


def define_out_of_process_context(python_file, fn_name, instance):
    check.inst_param(instance, 'instance', DagsterInstance)

    return DagsterGraphQLContext(
        locations=[
            PythonEnvRepositoryLocation(
                RepositoryLocationHandle.create_out_of_process_location(
                    'test_location', {fn_name: FileCodePointer(python_file, fn_name)}
                )
            )
        ],
        instance=instance,
    )


def define_context_for_repository_yaml(path, instance):
    check.inst_param(instance, 'instance', DagsterInstance)
    return DagsterGraphQLContext(
        locations=[
            InProcessRepositoryLocation(ReconstructableRepository.from_legacy_repository_yaml(path))
        ],
        instance=instance,
    )


def infer_repository(graphql_context):
    if len(graphql_context.repository_locations) == 1:
        # This is to account for having a single in process repository
        repository_location = graphql_context.repository_locations[0]
        repositories = repository_location.get_repositories()
        assert len(repositories) == 1
        return next(iter(repositories.values()))

    repository_location = graphql_context.get_repository_location('test')
    return repository_location.get_repository('test_repo')


def infer_repository_selector(graphql_context):
    if len(graphql_context.repository_locations) == 1:
        # This is to account for having a single in process repository
        repository_location = graphql_context.repository_locations[0]
        repositories = repository_location.get_repositories()
        assert len(repositories) == 1
        repository = next(iter(repositories.values()))
    else:
        repository_location = graphql_context.get_repository_location('test')
        repository = repository_location.get_repository('test_repo')

    return {
        'repositoryLocationName': repository_location.name,
        'repositoryName': repository.name,
    }


def infer_pipeline_selector(graphql_context, pipeline_name, solid_selection=None):
    selector = infer_repository_selector(graphql_context)
    selector.update({'pipelineName': pipeline_name, 'solidSelection': solid_selection})
    return selector


def infer_schedule_selector(graphql_context, schedule_name):
    selector = infer_repository_selector(graphql_context)
    selector.update({'scheduleName': schedule_name})
    return selector



from future.standard_library import install_aliases  # isort:skip

install_aliases()  # isort:skip

import weakref

import requests
from dagster_graphql.client.query import EXECUTE_RUN_IN_PROCESS_MUTATION
from requests import RequestException

from dagster import Bool, Field, check, seven
from dagster.core.errors import DagsterLaunchFailedError
from dagster.core.host_representation import ExternalPipeline
from dagster.core.launcher import RunLauncher
from dagster.core.storage.pipeline_run import PipelineRunStatus
from dagster.serdes import ConfigurableClass, ConfigurableClassData
from dagster.seven import urljoin, urlparse


class RemoteDagitRunLauncher(RunLauncher, ConfigurableClass):
    def __init__(self, address, timeout, inst_data=None):
        self._inst_data = check.opt_inst_param(inst_data, 'inst_data', ConfigurableClassData)
        self._address = check.str_param(address, 'address')
        self._timeout = check.numeric_param(timeout, 'timeout')
        self._handle = None
        self._instance_ref = None
        self._validated = False

        parsed_url = urlparse(address)
        check.invariant(
            parsed_url.scheme and parsed_url.netloc,
            'Address {address} is not a valid URL. Host URL should include scheme ie http://localhost'.format(
                address=self._address
            ),
        )

    @classmethod
    def config_type(cls):
        return {
            'address': str,
            'timeout': Field(float, is_required=False, default_value=30.0),
        }

    @classmethod
    def from_config_value(cls, inst_data, config_value):
        return cls(
            address=config_value['address'], timeout=config_value['timeout'], inst_data=inst_data,
        )

    @property
    def inst_data(self):
        return self._inst_data

    @property
    def _instance(self):
        return self._instance_ref() if self._instance_ref else None

    def start(self, handle):
        self._handle = handle

    def stop(self):
        self._handle = None

    def validate(self):
        if self._validated:
            return
        try:
            sanity_check = requests.get(
                urljoin(self._address, '/dagit_info'), timeout=self._timeout
            )
            self._validated = sanity_check.status_code = 200 and 'dagit' in sanity_check.text
        except RequestException:
            self._validated = False

        if not self._validated:
            raise DagsterLaunchFailedError(
                'Host {host} failed sanity check. It is not a dagit server.'.format(
                    host=self._address
                ),
            )

    def initialize(self, instance):
        # Store a weakref to avoid a circular reference / enable GC
        self._instance_ref = weakref.ref(instance)

    def launch_run(self, instance, run, external_pipeline):
        check.inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
        self.validate()

        variables = {
            'repositoryLocationName': external_pipeline.handle.location_name,
            'repositoryName': external_pipeline.handle.repository_name,
            'runId': run.run_id,
        }
        response = requests.post(
            urljoin(self._address, '/graphql'),
            params={
                'query': EXECUTE_RUN_IN_PROCESS_MUTATION,
                'variables': seven.json.dumps(variables),
            },
            timeout=self._timeout,
        )
        response.raise_for_status()
        result = response.json()['data']['executeRunInProcess']

        if result['__typename'] in ['LaunchPipelineRunSuccess', 'PipelineConfigValidationInvalid']:
            return self._instance.get_run_by_id(run.run_id)

        raise DagsterLaunchFailedError(
            'Failed to launch run with {cls} targeting {address}:\n{result}'.format(
                cls=self.__class__.__name__, address=self._address, result=result
            )
        )

    def can_terminate(self, run_id):
        return False

    def terminate(self, run_id):
        check.not_implemented('Termination not supported')



from dagster_graphql.test.utils import execute_dagster_graphql, infer_pipeline_selector

from .graphql_context_test_suite import ExecutingGraphQLContextTestMatrix
from .utils import sync_execute_get_run_log_data

COMPUTE_LOGS_QUERY = '''
  query ComputeLogsQuery($runId: ID!, $stepKey: String!) {
    pipelineRunOrError(runId: $runId) {
      ... on PipelineRun {
        runId
        computeLogs(stepKey: $stepKey) {
          stdout {
            data
          }
        }
      }
    }
  }
'''
COMPUTE_LOGS_SUBSCRIPTION = '''
  subscription ComputeLogsSubscription($runId: ID!, $stepKey: String!, $ioType: ComputeIOType!, $cursor: String!) {
    computeLogs(runId: $runId, stepKey: $stepKey, ioType: $ioType, cursor: $cursor) {
      data
    }
  }
'''


class TestComputeLogs(ExecutingGraphQLContextTestMatrix):
    def test_get_compute_logs_over_graphql(self, graphql_context, snapshot):
        selector = infer_pipeline_selector(graphql_context, 'spew_pipeline')
        payload = sync_execute_get_run_log_data(
            context=graphql_context,
            variables={'executionParams': {'selector': selector, 'mode': 'default'}},
        )
        run_id = payload['run']['runId']

        result = execute_dagster_graphql(
            graphql_context,
            COMPUTE_LOGS_QUERY,
            variables={'runId': run_id, 'stepKey': 'spew.compute'},
        )
        compute_logs = result.data['pipelineRunOrError']['computeLogs']
        snapshot.assert_match(compute_logs)

    def test_compute_logs_subscription_graphql(self, graphql_context, snapshot):
        selector = infer_pipeline_selector(graphql_context, 'spew_pipeline')
        payload = sync_execute_get_run_log_data(
            context=graphql_context,
            variables={'executionParams': {'selector': selector, 'mode': 'default'}},
        )
        run_id = payload['run']['runId']

        subscription = execute_dagster_graphql(
            graphql_context,
            COMPUTE_LOGS_SUBSCRIPTION,
            variables={
                'runId': run_id,
                'stepKey': 'spew.compute',
                'ioType': 'STDOUT',
                'cursor': '0',
            },
        )
        results = []
        subscription.subscribe(lambda x: results.append(x.data))
        assert len(results) == 1
        result = results[0]
        assert result['computeLogs']['data'] == 'HELLO WORLD\n'
        snapshot.assert_match(results)



from __future__ import absolute_import

from dagster_graphql import dauphin

from dagster import check
from dagster.config.errors import (
    EvaluationError,
    FieldNotDefinedErrorData,
    FieldsNotDefinedErrorData,
    MissingFieldErrorData,
    MissingFieldsErrorData,
    RuntimeMismatchErrorData,
    SelectorTypeErrorData,
)
from dagster.config.stack import EvaluationStackListItemEntry, EvaluationStackPathEntry
from dagster.core.host_representation import RepresentedPipeline
from dagster.core.snap import ConfigSchemaSnapshot
from dagster.utils.error import SerializableErrorInfo

from ..implementation.utils import PipelineSelector
from .config_types import DauphinConfigTypeField
from .runs import DauphinStepEvent


class DauphinError(dauphin.Interface):
    class Meta(object):
        name = 'Error'

    message = dauphin.String(required=True)


class DauphinPythonError(dauphin.ObjectType):
    class Meta(object):
        name = 'PythonError'
        interfaces = (DauphinError,)

    className = dauphin.Field(dauphin.String)
    stack = dauphin.non_null_list(dauphin.String)
    cause = dauphin.Field('PythonError')

    def __init__(self, error_info):
        super(DauphinPythonError, self).__init__()
        check.inst_param(error_info, 'error_info', SerializableErrorInfo)
        self.message = error_info.message
        self.stack = error_info.stack
        self.cause = error_info.cause
        self.className = error_info.cls_name


class DauphinSchedulerNotDefinedError(dauphin.ObjectType):
    class Meta(object):
        name = 'SchedulerNotDefinedError'
        interfaces = (DauphinError,)

    def __init__(self):
        super(DauphinSchedulerNotDefinedError, self).__init__()
        self.message = 'Scheduler is not defined for the currently loaded repository.'


class DauphinScheduleNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'ScheduleNotFoundError'
        interfaces = (DauphinError,)

    schedule_name = dauphin.NonNull(dauphin.String)

    def __init__(self, schedule_name):
        super(DauphinScheduleNotFoundError, self).__init__()
        self.schedule_name = check.str_param(schedule_name, 'schedule_name')
        self.message = (
            'Schedule {schedule_name} is not present in the currently loaded repository.'
        ).format(schedule_name=schedule_name)


class DauphinPipelineSnapshotNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'PipelineSnapshotNotFoundError'
        interfaces = (DauphinError,)

    snapshot_id = dauphin.NonNull(dauphin.String)

    def __init__(self, snapshot_id):
        super(DauphinPipelineSnapshotNotFoundError, self).__init__()
        self.snapshot_id = check.str_param(snapshot_id, 'snapshot_id')
        self.message = (
            'Pipeline snapshot {snapshot_id} is not present in the current instance.'
        ).format(snapshot_id=snapshot_id)


class DauphinReloadNotSupported(dauphin.ObjectType):
    class Meta(object):
        name = 'ReloadNotSupported'
        interfaces = (DauphinError,)

    def __init__(self, location_name):
        self.message = 'Location {location_name} does not support reloading.'.format(
            location_name=location_name
        )


class DauphinRepositoryLocationNotFound(dauphin.ObjectType):
    class Meta(object):
        name = 'RepositoryLocationNotFound'
        interfaces = (DauphinError,)

    def __init__(self, location_name):
        self.message = 'Location {location_name} does not exist.'.format(
            location_name=location_name
        )


class DauphinPipelineNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'PipelineNotFoundError'
        interfaces = (DauphinError,)

    pipeline_name = dauphin.NonNull(dauphin.String)
    repository_name = dauphin.NonNull(dauphin.String)
    repository_location_name = dauphin.NonNull(dauphin.String)

    def __init__(self, selector):
        super(DauphinPipelineNotFoundError, self).__init__()
        check.inst_param(selector, 'selector', PipelineSelector)
        self.pipeline_name = selector.pipeline_name
        self.repository_name = selector.repository_name
        self.repository_location_name = selector.location_name
        self.message = 'Could not find Pipeline {selector.location_name}.{selector.repository_name}.{selector.pipeline_name}'.format(
            selector=selector
        )


class DauphinPipelineRunNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'PipelineRunNotFoundError'
        interfaces = (DauphinError,)

    run_id = dauphin.NonNull(dauphin.String)

    def __init__(self, run_id):
        super(DauphinPipelineRunNotFoundError, self).__init__()
        self.run_id = check.str_param(run_id, 'run_id')
        self.message = 'Pipeline run {run_id} could not be found.'.format(run_id=run_id)


class DauphinInvalidPipelineRunsFilterError(dauphin.ObjectType):
    class Meta(object):
        name = 'InvalidPipelineRunsFilterError'
        interfaces = (DauphinError,)

    def __init__(self, message):
        super(DauphinInvalidPipelineRunsFilterError, self).__init__()
        self.message = check.str_param(message, 'message')


class DauphinRunGroupNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'RunGroupNotFoundError'
        interfaces = (DauphinError,)

    run_id = dauphin.NonNull(dauphin.String)

    def __init__(self, run_id):
        super(DauphinRunGroupNotFoundError, self).__init__()
        self.run_id = check.str_param(run_id, 'run_id')
        self.message = 'Run group of run {run_id} could not be found.'.format(run_id=run_id)


class DauphinInvalidSubsetError(dauphin.ObjectType):
    class Meta(object):
        name = 'InvalidSubsetError'
        interfaces = (DauphinError,)

    pipeline = dauphin.Field(dauphin.NonNull('Pipeline'))

    def __init__(self, message, pipeline):
        super(DauphinInvalidSubsetError, self).__init__()
        self.message = check.str_param(message, 'message')
        self.pipeline = pipeline


class DauphinPresetNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'PresetNotFoundError'
        interfaces = (DauphinError,)

    preset = dauphin.NonNull(dauphin.String)

    def __init__(self, preset, selector):
        self.preset = check.str_param(preset, 'preset')
        self.message = 'Preset {preset} not found in pipeline {pipeline}.'.format(
            preset=preset, pipeline=selector.pipeline_name
        )


class DauphinConflictingExecutionParamsError(dauphin.ObjectType):
    class Meta(object):
        name = 'ConflictingExecutionParamsError'
        interfaces = (DauphinError,)

    def __init__(self, conflicting_param):
        self.message = 'Invalid ExecutionParams. Cannot define {conflicting_param} when using a preset.'.format(
            conflicting_param=conflicting_param
        )


class DauphinModeNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'ModeNotFoundError'
        interfaces = (DauphinError,)

    mode = dauphin.NonNull(dauphin.String)

    def __init__(self, mode, selector):
        self.mode = check.str_param(mode, 'mode')
        self.message = 'Mode {mode} not found in pipeline {pipeline}.'.format(
            mode=mode, pipeline=selector.pipeline_name
        )


class DauphinPipelineConfigValidationValid(dauphin.ObjectType):
    class Meta(object):
        name = 'PipelineConfigValidationValid'

    pipeline_name = dauphin.NonNull(dauphin.String)


class DauphinPipelineConfigValidationInvalid(dauphin.ObjectType):
    class Meta(object):
        name = 'PipelineConfigValidationInvalid'

    pipeline_name = dauphin.NonNull(dauphin.String)
    errors = dauphin.non_null_list('PipelineConfigValidationError')

    @staticmethod
    def for_validation_errors(represented_pipeline, errors):
        check.inst_param(represented_pipeline, 'represented_pipeline', RepresentedPipeline)
        check.list_param(errors, 'errors', of_type=EvaluationError)
        return DauphinPipelineConfigValidationInvalid(
            pipeline_name=represented_pipeline.name,
            errors=[
                DauphinPipelineConfigValidationError.from_dagster_error(
                    represented_pipeline.config_schema_snapshot, err
                )
                for err in errors
            ],
        )


class DauphinPipelineConfigValidationResult(dauphin.Union):
    class Meta(object):
        name = 'PipelineConfigValidationResult'
        types = (
            DauphinInvalidSubsetError,
            DauphinPipelineConfigValidationValid,
            DauphinPipelineConfigValidationInvalid,
            DauphinPipelineNotFoundError,
            DauphinPythonError,
        )


class DauphinPipelineConfigValidationError(dauphin.Interface):
    class Meta(object):
        name = 'PipelineConfigValidationError'

    message = dauphin.NonNull(dauphin.String)
    path = dauphin.non_null_list(dauphin.String)
    stack = dauphin.NonNull('EvaluationStack')
    reason = dauphin.NonNull('EvaluationErrorReason')

    @staticmethod
    def from_dagster_error(config_schema_snapshot, error):
        check.inst_param(config_schema_snapshot, 'config_schema_snapshot', ConfigSchemaSnapshot)
        check.inst_param(error, 'error', EvaluationError)

        if isinstance(error.error_data, RuntimeMismatchErrorData):
            return DauphinRuntimeMismatchConfigError(
                message=error.message,
                path=[],  # TODO: remove
                stack=DauphinEvaluationStack(config_schema_snapshot, error.stack),
                reason=error.reason,
                value_rep=error.error_data.value_rep,
            )
        elif isinstance(error.error_data, MissingFieldErrorData):
            return DauphinMissingFieldConfigError(
                message=error.message,
                path=[],  # TODO: remove
                stack=DauphinEvaluationStack(config_schema_snapshot, error.stack),
                reason=error.reason,
                field=DauphinConfigTypeField(
                    config_schema_snapshot=config_schema_snapshot,
                    field_snap=error.error_data.field_snap,
                ),
            )
        elif isinstance(error.error_data, MissingFieldsErrorData):
            return DauphinMissingFieldsConfigError(
                message=error.message,
                path=[],  # TODO: remove
                stack=DauphinEvaluationStack(config_schema_snapshot, error.stack),
                reason=error.reason,
                fields=[
                    DauphinConfigTypeField(
                        config_schema_snapshot=config_schema_snapshot, field_snap=field_snap,
                    )
                    for field_snap in error.error_data.field_snaps
                ],
            )

        elif isinstance(error.error_data, FieldNotDefinedErrorData):
            return DauphinFieldNotDefinedConfigError(
                message=error.message,
                path=[],  # TODO: remove
                stack=DauphinEvaluationStack(config_schema_snapshot, error.stack),
                reason=error.reason,
                field_name=error.error_data.field_name,
            )
        elif isinstance(error.error_data, FieldsNotDefinedErrorData):
            return DauphinFieldsNotDefinedConfigError(
                message=error.message,
                path=[],  # TODO: remove
                stack=DauphinEvaluationStack(config_schema_snapshot, error.stack),
                reason=error.reason,
                field_names=error.error_data.field_names,
            )
        elif isinstance(error.error_data, SelectorTypeErrorData):
            return DauphinSelectorTypeConfigError(
                message=error.message,
                path=[],  # TODO: remove
                stack=DauphinEvaluationStack(config_schema_snapshot, error.stack),
                reason=error.reason,
                incoming_fields=error.error_data.incoming_fields,
            )
        else:
            check.failed(
                'Error type not supported {error_data}'.format(error_data=repr(error.error_data))
            )


class DauphinRuntimeMismatchConfigError(dauphin.ObjectType):
    class Meta(object):
        name = 'RuntimeMismatchConfigError'
        interfaces = (DauphinPipelineConfigValidationError,)

    value_rep = dauphin.Field(dauphin.String)


class DauphinMissingFieldConfigError(dauphin.ObjectType):
    class Meta(object):
        name = 'MissingFieldConfigError'
        interfaces = (DauphinPipelineConfigValidationError,)

    field = dauphin.NonNull('ConfigTypeField')


class DauphinMissingFieldsConfigError(dauphin.ObjectType):
    class Meta(object):
        name = 'MissingFieldsConfigError'
        interfaces = (DauphinPipelineConfigValidationError,)

    fields = dauphin.non_null_list('ConfigTypeField')


class DauphinFieldNotDefinedConfigError(dauphin.ObjectType):
    class Meta(object):
        name = 'FieldNotDefinedConfigError'
        interfaces = (DauphinPipelineConfigValidationError,)

    field_name = dauphin.NonNull(dauphin.String)


class DauphinFieldsNotDefinedConfigError(dauphin.ObjectType):
    class Meta(object):
        name = 'FieldsNotDefinedConfigError'
        interfaces = (DauphinPipelineConfigValidationError,)

    field_names = dauphin.non_null_list(dauphin.String)


class DauphinSelectorTypeConfigError(dauphin.ObjectType):
    class Meta(object):
        name = 'SelectorTypeConfigError'
        interfaces = (DauphinPipelineConfigValidationError,)

    incoming_fields = dauphin.non_null_list(dauphin.String)


class DauphinEvaluationErrorReason(dauphin.Enum):
    class Meta(object):
        name = 'EvaluationErrorReason'

    RUNTIME_TYPE_MISMATCH = 'RUNTIME_TYPE_MISMATCH'
    MISSING_REQUIRED_FIELD = 'MISSING_REQUIRED_FIELD'
    MISSING_REQUIRED_FIELDS = 'MISSING_REQUIRED_FIELDS'
    FIELD_NOT_DEFINED = 'FIELD_NOT_DEFINED'
    FIELDS_NOT_DEFINED = 'FIELDS_NOT_DEFINED'
    SELECTOR_FIELD_ERROR = 'SELECTOR_FIELD_ERROR'


class DauphinEvaluationStackListItemEntry(dauphin.ObjectType):
    class Meta(object):
        name = 'EvaluationStackListItemEntry'

    def __init__(self, list_index):
        super(DauphinEvaluationStackListItemEntry, self).__init__()
        self._list_index = list_index

    list_index = dauphin.NonNull(dauphin.Int)

    def resolve_list_index(self, _info):
        return self._list_index


class DauphinEvaluationStackPathEntry(dauphin.ObjectType):
    class Meta(object):
        name = 'EvaluationStackPathEntry'

    def __init__(self, field_name):
        self._field_name = check.str_param(field_name, 'field_name')
        super(DauphinEvaluationStackPathEntry, self).__init__()

    field_name = dauphin.NonNull(dauphin.String)

    def resolve_field_name(self, _info):
        return self._field_name


class DauphinEvaluationStackEntry(dauphin.Union):
    class Meta(object):
        name = 'EvaluationStackEntry'
        types = (DauphinEvaluationStackListItemEntry, DauphinEvaluationStackPathEntry)

    @staticmethod
    def from_native_entry(entry):
        if isinstance(entry, EvaluationStackPathEntry):
            return DauphinEvaluationStackPathEntry(field_name=entry.field_name)
        elif isinstance(entry, EvaluationStackListItemEntry):
            return DauphinEvaluationStackListItemEntry(list_index=entry.list_index)
        else:
            check.failed('Unsupported stack entry type {entry}'.format(entry=entry))


class DauphinEvaluationStack(dauphin.ObjectType):
    def __init__(self, config_schema_snapshot, stack):
        self._config_schema_snapshot = check.inst_param(
            config_schema_snapshot, 'config_schema_snapshot', ConfigSchemaSnapshot
        )
        self._stack = stack
        super(DauphinEvaluationStack, self).__init__()

    class Meta(object):
        name = 'EvaluationStack'

    entries = dauphin.non_null_list('EvaluationStackEntry')

    def resolve_entries(self, _):
        return map(DauphinEvaluationStackEntry.from_native_entry, self._stack.entries)


class DauphinExecutionPlanOrError(dauphin.Union):
    class Meta(object):
        name = 'ExecutionPlanOrError'
        types = (
            'ExecutionPlan',
            DauphinPipelineConfigValidationInvalid,
            DauphinPipelineNotFoundError,
            DauphinInvalidSubsetError,
            DauphinPythonError,
        )


class DauphinExecuteRunInProcessSuccess(dauphin.ObjectType):
    class Meta(object):
        name = 'ExecuteRunInProcessSuccess'

    run = dauphin.Field(dauphin.NonNull('PipelineRun'))


class DauphinLaunchPipelineRunSuccess(dauphin.ObjectType):
    class Meta(object):
        name = 'LaunchPipelineRunSuccess'

    run = dauphin.Field(dauphin.NonNull('PipelineRun'))


class DauphinInvalidStepError(dauphin.ObjectType):
    class Meta(object):
        name = 'InvalidStepError'

    invalid_step_key = dauphin.NonNull(dauphin.String)


class DauphinInvalidOutputError(dauphin.ObjectType):
    class Meta(object):
        name = 'InvalidOutputError'

    step_key = dauphin.NonNull(dauphin.String)
    invalid_output_name = dauphin.NonNull(dauphin.String)


class DauphinPipelineRunConflict(dauphin.ObjectType):
    class Meta(object):
        name = 'PipelineRunConflict'
        interfaces = (DauphinError,)


create_execution_params_error_types = (
    DauphinPresetNotFoundError,
    DauphinConflictingExecutionParamsError,
)

pipeline_execution_error_types = (
    DauphinInvalidStepError,
    DauphinInvalidOutputError,
    DauphinPipelineConfigValidationInvalid,
    DauphinPipelineNotFoundError,
    DauphinPipelineRunConflict,
    DauphinPythonError,
) + create_execution_params_error_types


launch_pipeline_run_result_types = (DauphinLaunchPipelineRunSuccess,)


class DauphinExecuteRunInProcessResult(dauphin.Union):
    class Meta(object):
        name = 'ExecuteRunInProcessResult'
        types = (
            (DauphinExecuteRunInProcessSuccess,)
            + pipeline_execution_error_types
            + (DauphinPipelineRunNotFoundError,)
        )


class DauphinLaunchPipelineExecutionResult(dauphin.Union):
    class Meta(object):
        name = 'LaunchPipelineExecutionResult'
        types = launch_pipeline_run_result_types + pipeline_execution_error_types


class DauphinExecutePlanSuccess(dauphin.ObjectType):
    class Meta(object):
        name = 'ExecutePlanSuccess'

    pipeline = dauphin.Field(dauphin.NonNull('Pipeline'))
    has_failures = dauphin.Field(dauphin.NonNull(dauphin.Boolean))
    step_events = dauphin.non_null_list(DauphinStepEvent)
    raw_event_records = dauphin.non_null_list(dauphin.String)


class DauphinExecutePlanResult(dauphin.Union):
    class Meta(object):
        name = 'ExecutePlanResult'
        types = (
            DauphinExecutePlanSuccess,
            DauphinPipelineConfigValidationInvalid,
            DauphinPipelineNotFoundError,
            DauphinInvalidStepError,
            DauphinPythonError,
        )


class DauphinLaunchPipelineReexecutionResult(dauphin.Union):
    class Meta(object):
        name = 'LaunchPipelineReexecutionResult'
        types = launch_pipeline_run_result_types + pipeline_execution_error_types


class DauphinConfigTypeNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'ConfigTypeNotFoundError'
        interfaces = (DauphinError,)

    pipeline = dauphin.NonNull('Pipeline')
    config_type_name = dauphin.NonNull(dauphin.String)


class DauphinDagsterTypeNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'DagsterTypeNotFoundError'
        interfaces = (DauphinError,)

    dagster_type_name = dauphin.NonNull(dauphin.String)


class DauphinConfigTypeOrError(dauphin.Union):
    class Meta(object):
        name = 'ConfigTypeOrError'
        types = (
            'EnumConfigType',
            'CompositeConfigType',
            'RegularConfigType',
            DauphinPipelineNotFoundError,
            DauphinConfigTypeNotFoundError,
            DauphinPythonError,
        )


class DauphinDagsterTypeOrError(dauphin.Union):
    class Meta(object):
        name = 'DagsterTypeOrError'
        types = (
            'RegularDagsterType',
            DauphinPipelineNotFoundError,
            DauphinDagsterTypeNotFoundError,
            DauphinPythonError,
        )


class DauphinPipelineRunOrError(dauphin.Union):
    class Meta(object):
        name = 'PipelineRunOrError'
        types = ('PipelineRun', DauphinPipelineRunNotFoundError, DauphinPythonError)


class DauphinPipelineRuns(dauphin.ObjectType):
    class Meta(object):
        name = 'PipelineRuns'

    results = dauphin.non_null_list('PipelineRun')


class DauphinPipelineRunsOrError(dauphin.Union):
    class Meta(object):
        name = 'PipelineRunsOrError'
        types = (DauphinPipelineRuns, DauphinInvalidPipelineRunsFilterError, DauphinPythonError)


class DauphinRunGroupOrError(dauphin.Union):
    class Meta(object):
        name = 'RunGroupOrError'
        types = ('RunGroup', DauphinRunGroupNotFoundError, DauphinPythonError)


class DauphinRunGroupsOrError(dauphin.ObjectType):
    class Meta(object):
        name = 'RunGroupsOrError'
        types = ('RunGroups', DauphinPythonError)

    results = dauphin.non_null_list('RunGroup')


class DauphinScheduleDefinitionNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'ScheduleDefinitionNotFoundError'
        interfaces = (DauphinError,)

    schedule_name = dauphin.NonNull(dauphin.String)

    def __init__(self, schedule_name):
        super(DauphinScheduleDefinitionNotFoundError, self).__init__()
        self.schedule_name = check.str_param(schedule_name, 'schedule_name')
        self.message = 'Schedule {schedule_name} could not be found.'.format(
            schedule_name=self.schedule_name
        )


class DauphinScheduleStateNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'ScheduleStateNotFoundError'
        interfaces = (DauphinError,)

    schedule_origin_id = dauphin.NonNull(dauphin.String)

    def __init__(self, schedule_origin_id):
        super(DauphinScheduleStateNotFoundError, self).__init__()
        self.schedule_origin_id = check.str_param(schedule_origin_id, 'schedule_origin_id')
        self.message = 'State for schedule {schedule_origin_id} could not be found.'.format(
            schedule_origin_id=self.schedule_origin_id
        )


class DauphinPartitionSetNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'PartitionSetNotFoundError'
        interfaces = (DauphinError,)

    partition_set_name = dauphin.NonNull(dauphin.String)

    def __init__(self, partition_set_name):
        super(DauphinPartitionSetNotFoundError, self).__init__()
        self.partition_set_name = check.str_param(partition_set_name, 'partition_set_name')
        self.message = 'Partition set {partition_set_name} could not be found.'.format(
            partition_set_name=self.partition_set_name
        )


class DauphinRepositoriesOrError(dauphin.Union):
    class Meta(object):
        name = 'RepositoriesOrError'
        types = ('RepositoryConnection', DauphinPythonError)


class DauphinRepositoryNotFoundError(dauphin.ObjectType):
    class Meta(object):
        name = 'RepositoryNotFoundError'
        interfaces = (DauphinError,)

    repository_name = dauphin.NonNull(dauphin.String)
    repository_location_name = dauphin.NonNull(dauphin.String)

    def __init__(self, repository_location_name, repository_name):
        super(DauphinRepositoryNotFoundError, self).__init__()
        self.repository_name = check.str_param(repository_name, 'repository_name')
        self.repository_location_name = check.str_param(
            repository_location_name, 'repository_location_name'
        )
        self.message = 'Could not find Repository {repository_location_name}.{repository_name}'.format(
            repository_name=repository_name, repository_location_name=repository_location_name
        )


class DauphinRepositoryOrError(dauphin.Union):
    class Meta(object):
        name = 'RepositoryOrError'
        types = (DauphinPythonError, 'Repository', DauphinRepositoryNotFoundError)



from dagster_graphql import dauphin
from dagster_graphql.implementation.execution import (
    ExecutionParams,
    delete_pipeline_run,
    do_execute_plan,
    get_compute_log_observable,
    get_pipeline_run_observable,
    launch_pipeline_execution,
    launch_pipeline_reexecution,
    terminate_pipeline_execution,
)
from dagster_graphql.implementation.execution.execute_run_in_process import (
    execute_run_in_graphql_process,
)
from dagster_graphql.implementation.external import (
    fetch_repositories,
    fetch_repository,
    get_full_external_pipeline_or_raise,
)
from dagster_graphql.implementation.fetch_assets import get_asset, get_assets
from dagster_graphql.implementation.fetch_partition_sets import (
    get_partition_set,
    get_partition_sets_or_error,
)
from dagster_graphql.implementation.fetch_pipelines import (
    get_pipeline_or_error,
    get_pipeline_snapshot_or_error_from_pipeline_selector,
    get_pipeline_snapshot_or_error_from_snapshot_id,
)
from dagster_graphql.implementation.fetch_runs import (
    get_execution_plan,
    get_run_by_id,
    get_run_group,
    get_run_groups,
    get_run_tags,
    get_runs,
    validate_pipeline_config,
)
from dagster_graphql.implementation.fetch_schedules import (
    get_schedule_definition_or_error,
    get_schedule_definitions_or_error,
    get_schedule_states_or_error,
    get_scheduler_or_error,
)
from dagster_graphql.implementation.run_config_schema import (
    resolve_is_run_config_valid,
    resolve_run_config_schema_or_error,
)
from dagster_graphql.implementation.utils import (
    ExecutionMetadata,
    UserFacingGraphQLError,
    capture_dauphin_error,
    pipeline_selector_from_graphql,
)

from dagster import check
from dagster.core.definitions.events import AssetKey
from dagster.core.execution.retries import Retries
from dagster.core.host_representation import (
    RepositorySelector,
    RepresentedPipeline,
    ScheduleSelector,
)
from dagster.core.instance import DagsterInstance
from dagster.core.launcher import RunLauncher
from dagster.core.storage.compute_log_manager import ComputeIOType
from dagster.core.storage.pipeline_run import PipelineRunStatus, PipelineRunsFilter

from .config_types import to_dauphin_config_type
from .runs import DauphinPipelineRunStatus
from .schedules import (
    DauphinReconcileSchedulerStateMutation,
    DauphinStartScheduleMutation,
    DauphinStopRunningScheduleMutation,
)


class DauphinQuery(dauphin.ObjectType):
    class Meta(object):
        name = 'Query'

    version = dauphin.NonNull(dauphin.String)

    repositoriesOrError = dauphin.NonNull('RepositoriesOrError')
    repositoryOrError = dauphin.Field(
        dauphin.NonNull('RepositoryOrError'),
        repositorySelector=dauphin.NonNull('RepositorySelector'),
    )

    pipelineOrError = dauphin.Field(
        dauphin.NonNull('PipelineOrError'), params=dauphin.NonNull('PipelineSelector')
    )

    pipelineSnapshotOrError = dauphin.Field(
        dauphin.NonNull('PipelineSnapshotOrError'),
        snapshotId=dauphin.String(),
        activePipelineSelector=dauphin.Argument('PipelineSelector'),
    )

    scheduler = dauphin.Field(dauphin.NonNull('SchedulerOrError'))

    scheduleDefinitionOrError = dauphin.Field(
        dauphin.NonNull('ScheduleDefinitionOrError'),
        schedule_selector=dauphin.NonNull('ScheduleSelector'),
    )
    scheduleDefinitionsOrError = dauphin.Field(
        dauphin.NonNull('ScheduleDefinitionsOrError'),
        repositorySelector=dauphin.NonNull('RepositorySelector'),
    )
    scheduleStatesOrError = dauphin.Field(
        dauphin.NonNull('ScheduleStatesOrError'),
        repositorySelector=dauphin.NonNull('RepositorySelector'),
        withNoScheduleDefinition=dauphin.Boolean(),
    )

    partitionSetsOrError = dauphin.Field(
        dauphin.NonNull('PartitionSetsOrError'),
        repositorySelector=dauphin.NonNull('RepositorySelector'),
        pipelineName=dauphin.NonNull(dauphin.String),
    )
    partitionSetOrError = dauphin.Field(
        dauphin.NonNull('PartitionSetOrError'),
        repositorySelector=dauphin.NonNull('RepositorySelector'),
        partitionSetName=dauphin.String(),
    )

    pipelineRunsOrError = dauphin.Field(
        dauphin.NonNull('PipelineRunsOrError'),
        filter=dauphin.Argument('PipelineRunsFilter'),
        cursor=dauphin.String(),
        limit=dauphin.Int(),
    )

    pipelineRunOrError = dauphin.Field(
        dauphin.NonNull('PipelineRunOrError'), runId=dauphin.NonNull(dauphin.ID)
    )

    pipelineRunTags = dauphin.non_null_list('PipelineTagAndValues')

    runGroupOrError = dauphin.Field(
        dauphin.NonNull('RunGroupOrError'), runId=dauphin.NonNull(dauphin.ID)
    )

    runGroupsOrError = dauphin.Field(
        dauphin.NonNull('RunGroupsOrError'),
        filter=dauphin.Argument('PipelineRunsFilter'),
        cursor=dauphin.String(),
        limit=dauphin.Int(),
    )

    isPipelineConfigValid = dauphin.Field(
        dauphin.NonNull('PipelineConfigValidationResult'),
        args={
            'pipeline': dauphin.Argument(dauphin.NonNull('PipelineSelector')),
            'runConfigData': dauphin.Argument('RunConfigData'),
            'mode': dauphin.Argument(dauphin.NonNull(dauphin.String)),
        },
    )

    executionPlanOrError = dauphin.Field(
        dauphin.NonNull('ExecutionPlanOrError'),
        args={
            'pipeline': dauphin.Argument(dauphin.NonNull('PipelineSelector')),
            'runConfigData': dauphin.Argument('RunConfigData'),
            'mode': dauphin.Argument(dauphin.NonNull(dauphin.String)),
        },
    )

    runConfigSchemaOrError = dauphin.Field(
        dauphin.NonNull('RunConfigSchemaOrError'),
        args={
            'selector': dauphin.Argument(dauphin.NonNull('PipelineSelector')),
            'mode': dauphin.Argument(dauphin.String),
        },
        description='''Fetch an environment schema given an execution selection and a mode.
        See the descripton on RunConfigSchema for more information.''',
    )

    instance = dauphin.NonNull('Instance')
    assetsOrError = dauphin.Field(dauphin.NonNull('AssetsOrError'))
    assetOrError = dauphin.Field(
        dauphin.NonNull('AssetOrError'),
        assetKey=dauphin.Argument(dauphin.NonNull('AssetKeyInput')),
    )

    def resolve_repositoriesOrError(self, graphene_info):
        return fetch_repositories(graphene_info)

    def resolve_repositoryOrError(self, graphene_info, **kwargs):
        return fetch_repository(
            graphene_info, RepositorySelector.from_graphql_input(kwargs.get('repositorySelector')),
        )

    def resolve_pipelineSnapshotOrError(self, graphene_info, **kwargs):
        snapshot_id_arg = kwargs.get('snapshotId')
        pipeline_selector_arg = kwargs.get('activePipelineSelector')
        check.invariant(
            not (snapshot_id_arg and pipeline_selector_arg),
            'Must only pass one of snapshotId or activePipelineSelector',
        )
        check.invariant(
            snapshot_id_arg or pipeline_selector_arg,
            'Must set one of snapshotId or activePipelineSelector',
        )

        if pipeline_selector_arg:
            pipeline_selector = pipeline_selector_from_graphql(
                graphene_info.context, kwargs['activePipelineSelector']
            )
            return get_pipeline_snapshot_or_error_from_pipeline_selector(
                graphene_info, pipeline_selector
            )
        else:
            return get_pipeline_snapshot_or_error_from_snapshot_id(graphene_info, snapshot_id_arg)

    def resolve_version(self, graphene_info):
        return graphene_info.context.version

    def resolve_scheduler(self, graphene_info):
        return get_scheduler_or_error(graphene_info)

    def resolve_scheduleDefinitionOrError(self, graphene_info, schedule_selector):
        return get_schedule_definition_or_error(
            graphene_info, ScheduleSelector.from_graphql_input(schedule_selector)
        )

    def resolve_scheduleDefinitionsOrError(self, graphene_info, **kwargs):
        return get_schedule_definitions_or_error(
            graphene_info, RepositorySelector.from_graphql_input(kwargs.get('repositorySelector'))
        )

    def resolve_scheduleStatesOrError(self, graphene_info, **kwargs):
        return get_schedule_states_or_error(
            graphene_info,
            RepositorySelector.from_graphql_input(kwargs.get('repositorySelector')),
            kwargs.get('withNoScheduleDefinition'),
        )

    def resolve_pipelineOrError(self, graphene_info, **kwargs):
        return get_pipeline_or_error(
            graphene_info, pipeline_selector_from_graphql(graphene_info.context, kwargs['params']),
        )

    def resolve_pipelineRunsOrError(self, graphene_info, **kwargs):
        filters = kwargs.get('filter')
        if filters is not None:
            filters = filters.to_selector()

        return graphene_info.schema.type_named('PipelineRuns')(
            results=get_runs(graphene_info, filters, kwargs.get('cursor'), kwargs.get('limit'))
        )

    def resolve_pipelineRunOrError(self, graphene_info, runId):
        return get_run_by_id(graphene_info, runId)

    def resolve_runGroupsOrError(self, graphene_info, **kwargs):
        filters = kwargs.get('filter')
        if filters is not None:
            filters = filters.to_selector()

        return graphene_info.schema.type_named('RunGroupsOrError')(
            results=get_run_groups(
                graphene_info, filters, kwargs.get('cursor'), kwargs.get('limit')
            )
        )

    def resolve_partitionSetsOrError(self, graphene_info, **kwargs):
        return get_partition_sets_or_error(
            graphene_info,
            RepositorySelector.from_graphql_input(kwargs.get('repositorySelector')),
            kwargs.get('pipelineName'),
        )

    def resolve_partitionSetOrError(self, graphene_info, **kwargs):
        return get_partition_set(
            graphene_info,
            RepositorySelector.from_graphql_input(kwargs.get('repositorySelector')),
            kwargs.get('partitionSetName'),
        )

    def resolve_pipelineRunTags(self, graphene_info):
        return get_run_tags(graphene_info)

    def resolve_runGroupOrError(self, graphene_info, runId):
        return get_run_group(graphene_info, runId)

    def resolve_isPipelineConfigValid(self, graphene_info, pipeline, **kwargs):
        return validate_pipeline_config(
            graphene_info,
            pipeline_selector_from_graphql(graphene_info.context, pipeline),
            kwargs.get('runConfigData'),
            kwargs.get('mode'),
        )

    def resolve_executionPlanOrError(self, graphene_info, pipeline, **kwargs):
        return get_execution_plan(
            graphene_info,
            pipeline_selector_from_graphql(graphene_info.context, pipeline),
            kwargs.get('runConfigData'),
            kwargs.get('mode'),
        )

    def resolve_runConfigSchemaOrError(self, graphene_info, **kwargs):
        return resolve_run_config_schema_or_error(
            graphene_info,
            pipeline_selector_from_graphql(graphene_info.context, kwargs['selector']),
            kwargs.get('mode'),
        )

    def resolve_instance(self, graphene_info):
        return graphene_info.schema.type_named('Instance')(graphene_info.context.instance)

    def resolve_assetsOrError(self, graphene_info):
        return get_assets(graphene_info)

    def resolve_assetOrError(self, graphene_info, **kwargs):
        return get_asset(graphene_info, AssetKey.from_graphql_input(kwargs['assetKey']))


class DauphinStepOutputHandle(dauphin.InputObjectType):
    class Meta(object):
        name = 'StepOutputHandle'

    stepKey = dauphin.NonNull(dauphin.String)
    outputName = dauphin.NonNull(dauphin.String)


class DauphinDeletePipelineRunSuccess(dauphin.ObjectType):
    class Meta(object):
        name = 'DeletePipelineRunSuccess'

    runId = dauphin.NonNull(dauphin.String)


class DauphinDeletePipelineRunResult(dauphin.Union):
    class Meta(object):
        name = 'DeletePipelineRunResult'
        types = ('DeletePipelineRunSuccess', 'PythonError', 'PipelineRunNotFoundError')


class DauphinDeleteRunMutation(dauphin.Mutation):
    class Meta(object):
        name = 'DeletePipelineRun'

    class Arguments(object):
        runId = dauphin.NonNull(dauphin.String)

    Output = dauphin.NonNull('DeletePipelineRunResult')

    def mutate(self, graphene_info, **kwargs):
        run_id = kwargs['runId']
        return delete_pipeline_run(graphene_info, run_id)


class DauphinTerminatePipelineExecutionSuccess(dauphin.ObjectType):
    class Meta(object):
        name = 'TerminatePipelineExecutionSuccess'

    run = dauphin.Field(dauphin.NonNull('PipelineRun'))


class DauphinTerminatePipelineExecutionFailure(dauphin.ObjectType):
    class Meta(object):
        name = 'TerminatePipelineExecutionFailure'

    run = dauphin.NonNull('PipelineRun')
    message = dauphin.NonNull(dauphin.String)


class DauphinTerminatePipelineExecutionResult(dauphin.Union):
    class Meta(object):
        name = 'TerminatePipelineExecutionResult'
        types = (
            'TerminatePipelineExecutionSuccess',
            'TerminatePipelineExecutionFailure',
            'PipelineRunNotFoundError',
            'PythonError',
        )


class DauphinExecuteRunInProcessMutation(dauphin.Mutation):
    class Meta(object):
        name = 'ExecuteRunInProcessMutation'
        description = (
            'Execute a pipeline run in the python environment '
            'dagit/dagster-graphql is currently operating in.'
        )

    class Arguments(object):
        run_id = dauphin.NonNull(dauphin.String)
        repositoryName = dauphin.NonNull(dauphin.String)
        repositoryLocationName = dauphin.NonNull(dauphin.String)

    Output = dauphin.NonNull('ExecuteRunInProcessResult')

    def mutate(self, graphene_info, run_id, repositoryLocationName, repositoryName):
        return execute_run_in_graphql_process(
            graphene_info,
            repository_location_name=repositoryLocationName,
            repository_name=repositoryName,
            run_id=run_id,
        )


@capture_dauphin_error
def create_execution_params_and_launch_pipeline_exec(graphene_info, execution_params_dict):
    # refactored into a helper function here in order to wrap with @capture_dauphin_error,
    # because create_execution_params may raise

    return launch_pipeline_execution(
        graphene_info,
        execution_params=create_execution_params(graphene_info, execution_params_dict),
    )


class DauphinLaunchPipelineExecutionMutation(dauphin.Mutation):
    class Meta(object):
        name = 'LaunchPipelineExecutionMutation'
        description = 'Launch a pipeline run via the run launcher configured on the instance.'

    class Arguments(object):
        executionParams = dauphin.NonNull('ExecutionParams')

    Output = dauphin.NonNull('LaunchPipelineExecutionResult')

    def mutate(self, graphene_info, **kwargs):
        return create_execution_params_and_launch_pipeline_exec(
            graphene_info, kwargs['executionParams']
        )


@capture_dauphin_error
def create_execution_params_and_launch_pipeline_reexec(graphene_info, execution_params_dict):
    # refactored into a helper function here in order to wrap with @capture_dauphin_error,
    # because create_execution_params may raise

    return launch_pipeline_reexecution(
        graphene_info,
        execution_params=create_execution_params(graphene_info, execution_params_dict),
    )


class DauphinLaunchPipelineReexecutionMutation(dauphin.Mutation):
    class Meta(object):
        name = 'DauphinLaunchPipelineReexecutionMutation'
        description = 'Re-launch a pipeline run via the run launcher configured on the instance'

    class Arguments(object):
        executionParams = dauphin.NonNull('ExecutionParams')

    Output = dauphin.NonNull('LaunchPipelineReexecutionResult')

    def mutate(self, graphene_info, **kwargs):
        return create_execution_params_and_launch_pipeline_reexec(
            graphene_info, execution_params_dict=kwargs['executionParams'],
        )


class DauphinTerminatePipelineExecutionMutation(dauphin.Mutation):
    class Meta(object):
        name = 'TerminatePipelineExecutionMutation'

    class Arguments(object):
        runId = dauphin.NonNull(dauphin.String)

    Output = dauphin.NonNull('TerminatePipelineExecutionResult')

    def mutate(self, graphene_info, **kwargs):
        return terminate_pipeline_execution(graphene_info, kwargs['runId'])


class DauphinReloadRepositoryLocationMutationResult(dauphin.Union):
    class Meta(object):
        name = 'ReloadRepositoryLocationMutationResult'
        types = ('RepositoryLocation', 'ReloadNotSupported', 'RepositoryLocationNotFound')


class DauphinReloadRepositoryLocationMutation(dauphin.Mutation):
    class Meta(object):
        name = 'ReloadRepositoryLocationMutation'

    class Arguments(object):
        repositoryLocationName = dauphin.NonNull(dauphin.String)

    Output = dauphin.NonNull('ReloadRepositoryLocationMutationResult')

    def mutate(self, graphene_info, **kwargs):
        location_name = kwargs['repositoryLocationName']

        if not graphene_info.context.has_repository_location(location_name):
            return graphene_info.schema.type_named('RepositoryLocationNotFound')(location_name)

        if not graphene_info.context.get_repository_location(location_name).is_reload_supported:
            return graphene_info.schema.type_named('ReloadNotSupported')(location_name)

        return graphene_info.schema.type_named('RepositoryLocation')(
            graphene_info.context.reload_repository_location(location_name)
        )


class DauphinExecutionTag(dauphin.InputObjectType):
    class Meta(object):
        name = 'ExecutionTag'

    key = dauphin.NonNull(dauphin.String)
    value = dauphin.NonNull(dauphin.String)


class DauphinMarshalledInput(dauphin.InputObjectType):
    class Meta(object):
        name = 'MarshalledInput'

    input_name = dauphin.NonNull(dauphin.String)
    key = dauphin.NonNull(dauphin.String)


class DauphinMarshalledOutput(dauphin.InputObjectType):
    class Meta(object):
        name = 'MarshalledOutput'

    output_name = dauphin.NonNull(dauphin.String)
    key = dauphin.NonNull(dauphin.String)


class DauphinStepExecution(dauphin.InputObjectType):
    class Meta(object):
        name = 'StepExecution'

    stepKey = dauphin.NonNull(dauphin.String)
    marshalledInputs = dauphin.List(dauphin.NonNull(DauphinMarshalledInput))
    marshalledOutputs = dauphin.List(dauphin.NonNull(DauphinMarshalledOutput))


class DauphinExecutionMetadata(dauphin.InputObjectType):
    class Meta(object):
        name = 'ExecutionMetadata'

    runId = dauphin.String()
    tags = dauphin.List(dauphin.NonNull(DauphinExecutionTag))
    rootRunId = dauphin.String()
    parentRunId = dauphin.String()


def create_execution_params(graphene_info, graphql_execution_params):
    preset_name = graphql_execution_params.get('preset')
    selector = pipeline_selector_from_graphql(
        graphene_info.context, graphql_execution_params['selector']
    )
    if preset_name:
        if graphql_execution_params.get('runConfigData'):
            raise UserFacingGraphQLError(
                graphene_info.schema.type_named('ConflictingExecutionParamsError')(
                    conflicting_param='runConfigData'
                )
            )

        if graphql_execution_params.get('mode'):
            raise UserFacingGraphQLError(
                graphene_info.schema.type_named('ConflictingExecutionParamsError')(
                    conflicting_param='mode'
                )
            )

        if selector.solid_selection:
            raise UserFacingGraphQLError(
                graphene_info.schema.type_named('ConflictingExecutionParamsError')(
                    conflicting_param='selector.solid_selection'
                )
            )

        external_pipeline = get_full_external_pipeline_or_raise(graphene_info, selector)

        if not external_pipeline.has_preset(preset_name):
            raise UserFacingGraphQLError(
                graphene_info.schema.type_named('PresetNotFoundError')(
                    preset=preset_name, selector=selector
                )
            )

        preset = external_pipeline.get_preset(preset_name)

        return ExecutionParams(
            selector=selector.with_solid_selection(preset.solid_selection),
            run_config=preset.run_config,
            mode=preset.mode,
            execution_metadata=create_execution_metadata(
                graphql_execution_params.get('executionMetadata')
            ),
            step_keys=graphql_execution_params.get('stepKeys'),
        )

    return execution_params_from_graphql(graphene_info.context, graphql_execution_params)


def execution_params_from_graphql(context, graphql_execution_params):
    return ExecutionParams(
        selector=pipeline_selector_from_graphql(context, graphql_execution_params.get('selector')),
        run_config=graphql_execution_params.get('runConfigData') or {},
        mode=graphql_execution_params.get('mode'),
        execution_metadata=create_execution_metadata(
            graphql_execution_params.get('executionMetadata')
        ),
        step_keys=graphql_execution_params.get('stepKeys'),
    )


def create_execution_metadata(graphql_execution_metadata):
    return (
        ExecutionMetadata(
            run_id=graphql_execution_metadata.get('runId'),
            tags={t['key']: t['value'] for t in graphql_execution_metadata.get('tags', [])},
            root_run_id=graphql_execution_metadata.get('rootRunId'),
            parent_run_id=graphql_execution_metadata.get('parentRunId'),
        )
        if graphql_execution_metadata
        else ExecutionMetadata(run_id=None, tags={})
    )


@capture_dauphin_error
def create_execution_params_and_do_exec_plan(graphene_info, execution_params_dict, retries_dict):
    # refactored into a helper function here in order to wrap with @capture_dauphin_error,
    # because create_execution_params may raise
    return do_execute_plan(
        graphene_info,
        execution_params=create_execution_params(graphene_info, execution_params_dict),
        retries=create_retries_params(retries_dict),
    )


class DauphinRetriesPreviousAttempts(dauphin.InputObjectType):
    class Meta(object):
        name = 'RetriesPreviousAttempts'

    key = dauphin.String()
    count = dauphin.Int()


class DauphinRetries(dauphin.InputObjectType):
    class Meta(object):
        name = 'Retries'

    mode = dauphin.Field(dauphin.String)
    retries_previous_attempts = dauphin.List(DauphinRetriesPreviousAttempts)


def create_retries_params(retries_config):
    return Retries.from_graphql_input(retries_config)


class DauphinExecutePlan(dauphin.Mutation):
    class Meta(object):
        name = 'ExecutePlan'

    class Arguments(object):
        executionParams = dauphin.NonNull('ExecutionParams')
        retries = dauphin.Argument('Retries')

    Output = dauphin.NonNull('ExecutePlanResult')

    def mutate(self, graphene_info, **kwargs):
        return create_execution_params_and_do_exec_plan(
            graphene_info,
            execution_params_dict=kwargs['executionParams'],
            retries_dict=kwargs['retries'] if 'retries' in kwargs else None,
        )


class DauphinMutation(dauphin.ObjectType):
    class Meta(object):
        name = 'Mutation'

    launch_pipeline_execution = DauphinLaunchPipelineExecutionMutation.Field()
    launch_pipeline_reexecution = DauphinLaunchPipelineReexecutionMutation.Field()
    reconcile_scheduler_state = DauphinReconcileSchedulerStateMutation.Field()
    start_schedule = DauphinStartScheduleMutation.Field()
    stop_running_schedule = DauphinStopRunningScheduleMutation.Field()
    terminate_pipeline_execution = DauphinTerminatePipelineExecutionMutation.Field()
    delete_pipeline_run = DauphinDeleteRunMutation.Field()
    reload_repository_location = DauphinReloadRepositoryLocationMutation.Field()

    # These below are never invoked by tools such as a dagit. They are in the
    # graphql layer because it was a convenient, pre-existing IPC layer.
    # so that components like run launchers and integrations with systems
    # like airflow could invoke runs and subplans across a process boundary.
    # They will eventually move to our "cli api" or its successor (possibly
    # grpc).

    execute_plan = DauphinExecutePlan.Field()
    execute_run_in_process = DauphinExecuteRunInProcessMutation.Field()


DauphinComputeIOType = dauphin.Enum.from_enum(ComputeIOType)


class DauphinSubscription(dauphin.ObjectType):
    class Meta(object):
        name = 'Subscription'

    pipelineRunLogs = dauphin.Field(
        dauphin.NonNull('PipelineRunLogsSubscriptionPayload'),
        runId=dauphin.Argument(dauphin.NonNull(dauphin.ID)),
        after=dauphin.Argument('Cursor'),
    )

    computeLogs = dauphin.Field(
        dauphin.NonNull('ComputeLogFile'),
        runId=dauphin.Argument(dauphin.NonNull(dauphin.ID)),
        stepKey=dauphin.Argument(dauphin.NonNull(dauphin.String)),
        ioType=dauphin.Argument(dauphin.NonNull('ComputeIOType')),
        cursor=dauphin.Argument(dauphin.String),
    )

    def resolve_pipelineRunLogs(self, graphene_info, runId, after=None):
        return get_pipeline_run_observable(graphene_info, runId, after)

    def resolve_computeLogs(self, graphene_info, runId, stepKey, ioType, cursor=None):
        check.str_param(ioType, 'ioType')  # need to resolve to enum
        return get_compute_log_observable(
            graphene_info, runId, stepKey, ComputeIOType(ioType), cursor
        )


class DauphinRunConfigData(dauphin.GenericScalar, dauphin.Scalar):
    class Meta(object):
        name = 'RunConfigData'
        description = '''This type is used when passing in a configuration object
        for pipeline configuration. This is any-typed in the GraphQL type system,
        but must conform to the constraints of the dagster config type system'''


class DauphinExecutionParams(dauphin.InputObjectType):
    class Meta(object):
        name = 'ExecutionParams'

    selector = dauphin.NonNull('PipelineSelector')
    runConfigData = dauphin.Field('RunConfigData')
    mode = dauphin.Field(dauphin.String)
    executionMetadata = dauphin.Field('ExecutionMetadata')
    stepKeys = dauphin.Field(dauphin.List(dauphin.NonNull(dauphin.String)))
    preset = dauphin.Field(dauphin.String)


class DauphinRepositorySelector(dauphin.InputObjectType):
    class Meta(object):
        name = 'RepositorySelector'
        description = '''This type represents the fields necessary to identify a repository.'''

    repositoryName = dauphin.NonNull(dauphin.String)
    repositoryLocationName = dauphin.NonNull(dauphin.String)


class DauphinPipelineSelector(dauphin.InputObjectType):
    class Meta(object):
        name = 'PipelineSelector'
        description = '''This type represents the fields necessary to identify a
        pipeline or pipeline subset.'''

    pipelineName = dauphin.NonNull(dauphin.String)
    repositoryName = dauphin.NonNull(dauphin.String)
    repositoryLocationName = dauphin.NonNull(dauphin.String)
    solidSelection = dauphin.List(dauphin.NonNull(dauphin.String))


class DauphinScheduleSelector(dauphin.InputObjectType):
    class Meta(object):
        name = 'ScheduleSelector'
        description = '''This type represents the fields necessary to identify a schedule.'''

    repositoryName = dauphin.NonNull(dauphin.String)
    repositoryLocationName = dauphin.NonNull(dauphin.String)
    scheduleName = dauphin.NonNull(dauphin.String)


class DauphinPipelineRunsFilter(dauphin.InputObjectType):
    class Meta(object):
        name = 'PipelineRunsFilter'
        description = '''This type represents a filter on pipeline runs.
        Currently, you may only pass one of the filter options.'''

    # Currently you may choose one of the following
    run_id = dauphin.Field(dauphin.String)
    pipeline_name = dauphin.Field(dauphin.String)
    tags = dauphin.List(dauphin.NonNull(DauphinExecutionTag))
    status = dauphin.Field(DauphinPipelineRunStatus)

    def to_selector(self):
        if self.status:
            status = PipelineRunStatus[self.status]
        else:
            status = None

        if self.tags:
            # We are wrapping self.tags in a list because dauphin.List is not marked as iterable
            tags = {tag['key']: tag['value'] for tag in list(self.tags)}
        else:
            tags = None

        run_ids = [self.run_id] if self.run_id else []
        return PipelineRunsFilter(
            run_ids=run_ids, pipeline_name=self.pipeline_name, tags=tags, status=status,
        )


class DauphinPipelineTagAndValues(dauphin.ObjectType):
    class Meta(object):
        name = 'PipelineTagAndValues'
        description = '''A run tag and the free-form values that have been associated
        with it so far.'''

    key = dauphin.NonNull(dauphin.String)
    values = dauphin.non_null_list(dauphin.String)


class DauphinRunConfigSchema(dauphin.ObjectType):
    def __init__(self, represented_pipeline, mode):
        self._represented_pipeline = check.inst_param(
            represented_pipeline, 'represented_pipeline', RepresentedPipeline
        )
        self._mode = check.str_param(mode, 'mode')

    class Meta(object):
        name = 'RunConfigSchema'
        description = '''The run config schema represents the all the config type
        information given a certain execution selection and mode of execution of that
        selection. All config interactions (e.g. checking config validity, fetching
        all config types, fetching in a particular config type) should be done
        through this type '''

    rootConfigType = dauphin.Field(
        dauphin.NonNull('ConfigType'),
        description='''Fetch the root environment type. Concretely this is the type that
        is in scope at the root of configuration document for a particular execution selection.
        It is the type that is in scope initially with a blank config editor.''',
    )
    allConfigTypes = dauphin.Field(
        dauphin.non_null_list('ConfigType'),
        description='''Fetch all the named config types that are in the schema. Useful
        for things like a type browser UI, or for fetching all the types are in the
        scope of a document so that the index can be built for the autocompleting editor.
    ''',
    )

    isRunConfigValid = dauphin.Field(
        dauphin.NonNull('PipelineConfigValidationResult'),
        args={'runConfigData': dauphin.Argument('RunConfigData')},
        description='''Parse a particular environment config result. The return value
        either indicates that the validation succeeded by returning
        `PipelineConfigValidationValid` or that there are configuration errors
        by returning `PipelineConfigValidationInvalid' which containers a list errors
        so that can be rendered for the user''',
    )

    def resolve_allConfigTypes(self, _graphene_info):
        return sorted(
            list(
                map(
                    lambda key: to_dauphin_config_type(
                        self._represented_pipeline.config_schema_snapshot, key
                    ),
                    self._represented_pipeline.config_schema_snapshot.all_config_keys,
                )
            ),
            key=lambda ct: ct.key,
        )

    def resolve_rootConfigType(self, _graphene_info):
        return to_dauphin_config_type(
            self._represented_pipeline.config_schema_snapshot,
            self._represented_pipeline.get_mode_def_snap(self._mode).root_config_key,
        )

    def resolve_isRunConfigValid(self, graphene_info, **kwargs):
        return resolve_is_run_config_valid(
            graphene_info, self._represented_pipeline, self._mode, kwargs.get('runConfigData', {}),
        )


class DauphinRunConfigSchemaOrError(dauphin.Union):
    class Meta(object):
        name = 'RunConfigSchemaOrError'
        types = (
            'RunConfigSchema',
            'PipelineNotFoundError',
            'InvalidSubsetError',
            'ModeNotFoundError',
            'PythonError',
        )


class DauphinRunLauncher(dauphin.ObjectType):
    class Meta(object):
        name = 'RunLauncher'

    name = dauphin.NonNull(dauphin.String)

    def __init__(self, run_launcher):
        self._run_launcher = check.inst_param(run_launcher, 'run_launcher', RunLauncher)

    def resolve_name(self, _graphene_info):
        return self._run_launcher.__class__.__name__


class DauphinInstance(dauphin.ObjectType):
    class Meta(object):
        name = 'Instance'

    info = dauphin.NonNull(dauphin.String)
    runLauncher = dauphin.Field('RunLauncher')
    disableRunStart = dauphin.NonNull(dauphin.Boolean)
    assetsSupported = dauphin.NonNull(dauphin.Boolean)

    def __init__(self, instance):
        self._instance = check.inst_param(instance, 'instance', DagsterInstance)

    def resolve_info(self, _graphene_info):
        return self._instance.info_str()

    def resolve_runLauncher(self, _graphene_info):
        return (
            DauphinRunLauncher(self._instance.run_launcher) if self._instance.run_launcher else None
        )

    def resolve_disableRunStart(self, _graphene_info):
        execution_manager_settings = self._instance.dagit_settings.get('execution_manager')
        if not execution_manager_settings:
            return False
        return execution_manager_settings.get('disabled', False)

    def resolve_assetsSupported(self, _graphene_info):
        return self._instance.is_asset_aware


class DauphinAssetKeyInput(dauphin.InputObjectType):
    class Meta(object):
        name = 'AssetKeyInput'

    path = dauphin.non_null_list(dauphin.String)



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots['test_success_whole_execution_plan_with_in_memory_config 1'] = {
    'executePlan': {
        '__typename': 'ExecutePlanSuccess',
        'hasFailures': False,
        'pipeline': {
            'name': 'csv_hello_world'
        },
        'stepEvents': [
            {
                '__typename': 'ExecutionStepStartEvent',
                'message': 'Started execution of step "sum_solid.compute".',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepInputEvent',
                'message': 'Got input "num" of type "PoorMansDataFrame". (Type check passed).',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepOutputEvent',
                'message': 'Yielded output "result" of type "PoorMansDataFrame". (Type check passed).',
                'outputName': 'result',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepSuccessEvent',
                'message': 'Finished execution of step "sum_solid.compute" in {N}ms.',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepStartEvent',
                'message': 'Started execution of step "sum_sq_solid.compute".',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ExecutionStepInputEvent',
                'message': 'Got input "sum_df" of type "PoorMansDataFrame". (Type check passed).',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ExecutionStepOutputEvent',
                'message': 'Yielded output "result" of type "PoorMansDataFrame". (Type check passed).',
                'outputName': 'result',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ExecutionStepSuccessEvent',
                'message': 'Finished execution of step "sum_sq_solid.compute" in {N}ms.',
                'stepKey': 'sum_sq_solid.compute'
            }
        ]
    }
}

snapshots['test_successful_one_part_execute_plan 1'] = {
    'executePlan': {
        '__typename': 'ExecutePlanSuccess',
        'hasFailures': False,
        'pipeline': {
            'name': 'csv_hello_world'
        },
        'stepEvents': [
            {
                '__typename': 'ExecutionStepStartEvent',
                'message': 'Started execution of step "sum_solid.compute".',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepInputEvent',
                'message': 'Got input "num" of type "PoorMansDataFrame". (Type check passed).',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepOutputEvent',
                'message': 'Yielded output "result" of type "PoorMansDataFrame". (Type check passed).',
                'outputName': 'result',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ObjectStoreOperationEvent',
                'message': 'Stored intermediate object for output result in filesystem object store using pickle.',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepSuccessEvent',
                'message': 'Finished execution of step "sum_solid.compute" in {N}ms.',
                'stepKey': 'sum_solid.compute'
            }
        ]
    }
}

snapshots['test_successful_two_part_execute_plan 1'] = {
    'executePlan': {
        '__typename': 'ExecutePlanSuccess',
        'hasFailures': False,
        'pipeline': {
            'name': 'csv_hello_world'
        },
        'stepEvents': [
            {
                '__typename': 'ExecutionStepStartEvent',
                'message': 'Started execution of step "sum_solid.compute".',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepInputEvent',
                'message': 'Got input "num" of type "PoorMansDataFrame". (Type check passed).',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepOutputEvent',
                'message': 'Yielded output "result" of type "PoorMansDataFrame". (Type check passed).',
                'outputName': 'result',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ObjectStoreOperationEvent',
                'message': 'Stored intermediate object for output result in filesystem object store using pickle.',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepSuccessEvent',
                'message': 'Finished execution of step "sum_solid.compute" in {N}ms.',
                'stepKey': 'sum_solid.compute'
            }
        ]
    }
}

snapshots['test_successful_two_part_execute_plan 2'] = {
    'executePlan': {
        '__typename': 'ExecutePlanSuccess',
        'hasFailures': False,
        'pipeline': {
            'name': 'csv_hello_world'
        },
        'stepEvents': [
            {
                '__typename': 'ExecutionStepStartEvent',
                'message': 'Started execution of step "sum_sq_solid.compute".',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ObjectStoreOperationEvent',
                'message': 'Retrieved intermediate object for input sum_df in filesystem object store using pickle.',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ExecutionStepInputEvent',
                'message': 'Got input "sum_df" of type "PoorMansDataFrame". (Type check passed).',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ExecutionStepOutputEvent',
                'message': 'Yielded output "result" of type "PoorMansDataFrame". (Type check passed).',
                'outputName': 'result',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ObjectStoreOperationEvent',
                'message': 'Stored intermediate object for output result in filesystem object store using pickle.',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ExecutionStepSuccessEvent',
                'message': 'Finished execution of step "sum_sq_solid.compute" in {N}ms.',
                'stepKey': 'sum_sq_solid.compute'
            }
        ]
    }
}

snapshots['TestExecutionPlan.test_invalid_config_fetch_execute_plan[readonly_in_memory_instance_in_process_env] 1'] = {
    'executionPlanOrError': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}

snapshots['TestExecutionPlan.test_invalid_config_fetch_execute_plan[readonly_in_memory_instance_out_of_process_env] 1'] = {
    'executionPlanOrError': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}

snapshots['TestExecutionPlan.test_invalid_config_fetch_execute_plan[readonly_in_memory_instance_multi_location] 1'] = {
    'executionPlanOrError': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}

snapshots['TestExecutionPlan.test_invalid_config_fetch_execute_plan[readonly_sqlite_instance_in_process_env] 1'] = {
    'executionPlanOrError': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}

snapshots['TestExecutionPlan.test_invalid_config_fetch_execute_plan[readonly_sqlite_instance_out_of_process_env] 1'] = {
    'executionPlanOrError': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}

snapshots['TestExecutionPlan.test_invalid_config_fetch_execute_plan[readonly_sqlite_instance_multi_location] 1'] = {
    'executionPlanOrError': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}

snapshots['TestExecutionPlan.test_invalid_config_fetch_execute_plan[readonly_postgres_instance_in_process_env] 1'] = {
    'executionPlanOrError': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}

snapshots['TestExecutionPlan.test_invalid_config_fetch_execute_plan[readonly_postgres_instance_out_of_process_env] 1'] = {
    'executionPlanOrError': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}

snapshots['TestExecutionPlan.test_invalid_config_fetch_execute_plan[readonly_postgres_instance_multi_location] 1'] = {
    'executionPlanOrError': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}

snapshots['test_invalid_config_execute_plan 1'] = {
    'executePlan': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}

snapshots['test_pipeline_not_found_error_execute_plan 1'] = {
    'executePlan': {
        '__typename': 'PipelineNotFoundError',
        'pipelineName': 'nope'
    }
}

snapshots['test_success_whole_execution_plan 1'] = {
    'executePlan': {
        '__typename': 'ExecutePlanSuccess',
        'hasFailures': False,
        'pipeline': {
            'name': 'csv_hello_world'
        },
        'stepEvents': [
            {
                '__typename': 'ExecutionStepStartEvent',
                'message': 'Started execution of step "sum_solid.compute".',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepInputEvent',
                'message': 'Got input "num" of type "PoorMansDataFrame". (Type check passed).',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepOutputEvent',
                'message': 'Yielded output "result" of type "PoorMansDataFrame". (Type check passed).',
                'outputName': 'result',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ObjectStoreOperationEvent',
                'message': 'Stored intermediate object for output result in filesystem object store using pickle.',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepSuccessEvent',
                'message': 'Finished execution of step "sum_solid.compute" in {N}ms.',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepStartEvent',
                'message': 'Started execution of step "sum_sq_solid.compute".',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ObjectStoreOperationEvent',
                'message': 'Retrieved intermediate object for input sum_df in filesystem object store using pickle.',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ExecutionStepInputEvent',
                'message': 'Got input "sum_df" of type "PoorMansDataFrame". (Type check passed).',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ExecutionStepOutputEvent',
                'message': 'Yielded output "result" of type "PoorMansDataFrame". (Type check passed).',
                'outputName': 'result',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ObjectStoreOperationEvent',
                'message': 'Stored intermediate object for output result in filesystem object store using pickle.',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ExecutionStepSuccessEvent',
                'message': 'Finished execution of step "sum_sq_solid.compute" in {N}ms.',
                'stepKey': 'sum_sq_solid.compute'
            }
        ]
    }
}

snapshots['test_success_whole_execution_plan_with_filesystem_config 1'] = {
    'executePlan': {
        '__typename': 'ExecutePlanSuccess',
        'hasFailures': False,
        'pipeline': {
            'name': 'csv_hello_world'
        },
        'stepEvents': [
            {
                '__typename': 'ExecutionStepStartEvent',
                'message': 'Started execution of step "sum_solid.compute".',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepInputEvent',
                'message': 'Got input "num" of type "PoorMansDataFrame". (Type check passed).',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepOutputEvent',
                'message': 'Yielded output "result" of type "PoorMansDataFrame". (Type check passed).',
                'outputName': 'result',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ObjectStoreOperationEvent',
                'message': 'Stored intermediate object for output result in filesystem object store using pickle.',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepSuccessEvent',
                'message': 'Finished execution of step "sum_solid.compute" in {N}ms.',
                'stepKey': 'sum_solid.compute'
            },
            {
                '__typename': 'ExecutionStepStartEvent',
                'message': 'Started execution of step "sum_sq_solid.compute".',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ObjectStoreOperationEvent',
                'message': 'Retrieved intermediate object for input sum_df in filesystem object store using pickle.',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ExecutionStepInputEvent',
                'message': 'Got input "sum_df" of type "PoorMansDataFrame". (Type check passed).',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ExecutionStepOutputEvent',
                'message': 'Yielded output "result" of type "PoorMansDataFrame". (Type check passed).',
                'outputName': 'result',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ObjectStoreOperationEvent',
                'message': 'Stored intermediate object for output result in filesystem object store using pickle.',
                'stepKey': 'sum_sq_solid.compute'
            },
            {
                '__typename': 'ExecutionStepSuccessEvent',
                'message': 'Finished execution of step "sum_sq_solid.compute" in {N}ms.',
                'stepKey': 'sum_sq_solid.compute'
            }
        ]
    }
}

snapshots['TestExecutionPlan.test_invalid_config_fetch_execute_plan[readonly_in_memory_instance_grpc] 1'] = {
    'executionPlanOrError': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}

snapshots['TestExecutionPlan.test_invalid_config_fetch_execute_plan[readonly_sqlite_instance_external_grpc_server] 1'] = {
    'executionPlanOrError': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}

snapshots['TestExecutionPlan.test_invalid_config_fetch_execute_plan[readonly_sqlite_instance_grpc] 1'] = {
    'executionPlanOrError': {
        '__typename': 'PipelineConfigValidationInvalid',
        'errors': [
            {
                'message': 'Invalid scalar at path root:solids:sum_solid:inputs:num'
            }
        ],
        'pipelineName': 'csv_hello_world'
    }
}



from dagster.core.scheduler import ScheduleTickStatus


def tick_specific_data_from_dagster_tick(graphene_info, tick):
    if tick.status == ScheduleTickStatus.SUCCESS:
        run_id = tick.run_id
        run = None
        if graphene_info.context.instance.has_run(run_id):
            run = graphene_info.schema.type_named('PipelineRun')(
                graphene_info.context.instance.get_run_by_id(run_id)
            )
        return graphene_info.schema.type_named('ScheduleTickSuccessData')(run=run)
    elif tick.status == ScheduleTickStatus.FAILURE:
        error = tick.error
        return graphene_info.schema.type_named('ScheduleTickFailureData')(error=error)



from dagster.core.code_pointer import FileCodePointer
from dagster.core.host_representation import PythonEnvRepositoryLocation, RepositoryLocationHandle
from dagster.utils import file_relative_path


def test_dagster_out_of_process_location():
    env = PythonEnvRepositoryLocation(
        RepositoryLocationHandle.create_out_of_process_location(
            location_name='test_location',
            repository_code_pointer_dict={
                'test_repo': FileCodePointer(file_relative_path(__file__, 'setup.py'), 'test_repo'),
            },
        )
    )
    assert env.get_repository('test_repo')



import yaml
from graphql.execution.base import ResolveInfo

from dagster import check
from dagster.core.host_representation import (
    ExternalPartitionConfigData,
    ExternalPartitionExecutionErrorData,
    ExternalPartitionNamesData,
    ExternalPartitionSet,
    ExternalPartitionTagsData,
    RepositoryHandle,
    RepositorySelector,
)

from .utils import capture_dauphin_error


def get_partition_sets_or_error(graphene_info, repository_selector, pipeline_name):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(repository_selector, 'repository_selector', RepositorySelector)
    check.str_param(pipeline_name, 'pipeline_name')
    location = graphene_info.context.get_repository_location(repository_selector.location_name)
    repository = location.get_repository(repository_selector.repository_name)
    partition_sets = [
        partition_set
        for partition_set in repository.get_external_partition_sets()
        if partition_set.pipeline_name == pipeline_name
    ]

    return graphene_info.schema.type_named('PartitionSets')(
        results=[
            graphene_info.schema.type_named('PartitionSet')(
                external_repository_handle=repository.handle, external_partition_set=partition_set,
            )
            for partition_set in sorted(
                partition_sets,
                key=lambda partition_set: (
                    partition_set.pipeline_name,
                    partition_set.mode,
                    partition_set.name,
                ),
            )
        ]
    )


@capture_dauphin_error
def get_partition_set(graphene_info, repository_selector, partition_set_name):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(repository_selector, 'repository_selector', RepositorySelector)
    check.str_param(partition_set_name, 'partition_set_name')
    location = graphene_info.context.get_repository_location(repository_selector.location_name)
    repository = location.get_repository(repository_selector.repository_name)
    partition_sets = repository.get_external_partition_sets()
    for partition_set in partition_sets:
        if partition_set.name == partition_set_name:
            return graphene_info.schema.type_named('PartitionSet')(
                external_repository_handle=repository.handle, external_partition_set=partition_set,
            )

    return graphene_info.schema.type_named('PartitionSetNotFoundError')(partition_set_name)


@capture_dauphin_error
def get_partition_by_name(graphene_info, repository_handle, partition_set, partition_name):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
    check.inst_param(partition_set, 'partition_set', ExternalPartitionSet)
    check.str_param(partition_name, 'partition_name')
    return graphene_info.schema.type_named('Partition')(
        external_repository_handle=repository_handle,
        external_partition_set=partition_set,
        partition_name=partition_name,
    )


def get_partition_config(graphene_info, repository_handle, partition_set_name, partition_name):
    check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
    check.str_param(partition_set_name, 'partition_set_name')
    check.str_param(partition_name, 'partition_name')

    result = graphene_info.context.get_external_partition_config(
        repository_handle, partition_set_name, partition_name,
    )

    if isinstance(result, ExternalPartitionConfigData):
        return graphene_info.schema.type_named('PartitionRunConfig')(
            yaml=yaml.safe_dump(result.run_config, default_flow_style=False)
        )
    else:
        return graphene_info.schema.type_named('PythonError')(result.error)


def get_partition_tags(graphene_info, repository_handle, partition_set_name, partition_name):
    check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
    check.str_param(partition_set_name, 'partition_set_name')
    check.str_param(partition_name, 'partition_name')

    result = graphene_info.context.get_external_partition_tags(
        repository_handle, partition_set_name, partition_name
    )

    if isinstance(result, ExternalPartitionTagsData):
        return graphene_info.schema.type_named('PartitionTags')(
            results=[
                graphene_info.schema.type_named('PipelineTag')(key=key, value=value)
                for key, value in result.tags.items()
            ]
        )
    else:
        return graphene_info.schema.type_named('PythonError')(result.error)


def get_partitions(
    graphene_info, repository_handle, partition_set, cursor=None, limit=None, reverse=False
):
    check.inst_param(repository_handle, 'repository_handle', RepositoryHandle)
    check.inst_param(partition_set, 'partition_set', ExternalPartitionSet)
    result = graphene_info.context.get_external_partition_names(
        repository_handle, partition_set.name
    )

    if isinstance(result, ExternalPartitionNamesData):
        partition_names = _apply_cursor_limit_reverse(
            result.partition_names, cursor, limit, reverse
        )

        return graphene_info.schema.type_named('Partitions')(
            results=[
                graphene_info.schema.type_named('Partition')(
                    external_partition_set=partition_set,
                    external_repository_handle=repository_handle,
                    partition_name=partition_name,
                )
                for partition_name in partition_names
            ]
        )

    else:
        assert isinstance(result, ExternalPartitionExecutionErrorData)
        return graphene_info.schema.type_named('PythonError')(result.error)


def _apply_cursor_limit_reverse(items, cursor, limit, reverse):
    start = 0
    end = len(items)
    index = 0

    if cursor:
        index = next((idx for (idx, item) in enumerate(items) if item == cursor), None)

        if reverse:
            end = index
        else:
            start = index + 1

    if limit:
        if reverse:
            start = end - limit
        else:
            end = start + limit

    return items[start:end]
